## 25.10. added if (p0==1) v <- as.matrix(V)
##        (in new version of R, var(.) returns matrix even for dim=1
##        making this change necessary



ppmx <-  function(Yin=fd,mis=mis,p0=1,p1=0,p2=9,q2=NULL,p4=0,
                      censoring=0,   # indicator for censoring of response
                                     ## only for p0=1
                      cov.dahl=0,    # indicator for using Dahl model
                      cov.Vj1=NULL,  # variance of aux model for cont cov
                      cov.mean1=NULL,# prior mean & var for location 
                      cov.B1=NULL,   #    par of auxiliary for cont cov
                      cov.std1=1,    # indicator for standardizing cont covs
                      cov.pi2=NULL,
                      cov.a4=NULL,
                      cov.b4=NULL,
                      n.iter=4000,    # n MCMC iterations
                      n.discard=3000,  # initial transient
                      n.reinit=10000, # reinitialize
                      n.batch=50,     # batch size to thin out
                      n.predupdate=100, # batch size to update predictives
                      n.printallpars=1000, # batch size to print all pars 
                      m.prior=0,      # resample m
                      B.prior=0,      # resample B
                      S.prior=0,      # resample B
                      verbose=1,      # level of comments
                      py=1,           # marginal posterior predictive
                      pyvec=0,        # posterior predictive sim 
                      iy=NULL,        # index for y
                      ny=50,          # grid size for y
                      ygrid=NULL,     # (min,max) of ygrid
                      nx=NULL,        # number of cov-combinations
                      xgrid=NULL,     # actual cov-combinations
                      s=15,           # d.f. for prior on S
                      S.init=NULL,    # initial value and prior mean for S
                      qq=5,           # d.f. for prior on R
                      R=NULL,         # hyperprior parameter R
                      B.scale=100,    # initial value for B = B.scale*S.init,
                      cc=5,           # d.f. par c
                      m.init=NULL,    # initial values for m
                      a=NULL,         # hyperpar a
                      A.inv=NULL,     # hyperpar A
                      alpha=1,        # initial value for total mass alpha
                      a0=1,
                      b0=1,           # hyperpr pars for alpha ~ IG(a0/2,b0/2)
                      k0=NULL,
                      bal=.2,
                      ab=.1,
                      cd=.1)        # initial number of clusters
                      
  { # read in data
    if (is.null(Yin)){
      cat("\n *** Error: need data matrix or file Yin.\n")
      return(-1)
    }
    if(is.character(Yin)){
      if (!file.exists(Yin)){ ## reading does not work
        cat("\n *** Error: could not read data from file",Yin,
            "\n Hint: need to include full path.\n")
        return(-1)
      }
      Y <- read.table(Yin,header=T)
      if (verbose>0){
        cat("Read ", nrow(Y), " rows and ", ncol(Y), " columns",
            "from data file ", Yin,"\n")
        cat("Variable (column) names are: ",names(Y),"\n")
      }
    } else { # not char
      if ( (!is.matrix(Yin)) & (!is.data.frame(Yin))){
        cat("\n *** Error: need Yin = filename or matrix or data frame.\n")
        return(-1)
      }
      Y <- Yin
    }
    if (is.null(p0) | is.null(p1) | is.null(p2) | is.null(p4)){
        cat("\n *** Error: need to set p0,p1,p2,p4.\n")
        return(-1)
      }
      
    p <- ncol(Y)
    n <- nrow(Y)
    q <- apply(Y,2,n.unique) # will use later..
    if (p0+p1+p2+p4 != p){
      cat("\n *** Error: n columns != p0+p1+p2+p4.\n")
      return(-1)
    }
    ## continuous covariates: parameters and standardize
    if (p1>0){
      idx1 <- (p0+1):(p0+p1)
      if (is.null(cov.Vj1))
        cov.Vj1 <- rep(0.25,p1)
      if (is.null(cov.mean1))
        cov.mean1 <- rep(0,p1)
      if (is.null(cov.B1))
        cov.B1 <- rep(1,p1)
      if ((length(cov.Vj1)!=p1) | (length(cov.mean1)!=p1) |
          (length(cov.B1)!=p1 ) ){
        cat("\n *** Error: cov.Vj1, mean1 and B1 need to be of length p1.\n")
        return(-1)
      }

      if (any(q[idx1] <= 3))
        cat("\n *** Warning: check that columns", (p0+1), "through", (p0+p1),"\n",
            "    are really continous covariates. Only",q[idx1],
            " unique values.\n")
      if (cov.std1==1){
        Y1 <- as.matrix( Y[,idx1] ) # as.matrix in case p1=1
        v1 <- apply(Y1,2,var)       # empirical var of cont cov's
        s1 <- sqrt(v1)
        m1 <- apply(Y1,2,mean)      # empirical mean
        y1 <- t( (t(Y1) - m1)/s1 )  # standardized continuous vars
        Y[,idx1] <- y1               # substitute Y
      }
    }
    ## categorical covariates
    if (p2>0){
      idx2 <- (p0+p1+1):(p0+p1+p2)
      q2 <- q[idx2]
      if (any(q2 > 10))
        cat("\n *** Warning: check that cols (p0+p1+1...p0+p1+p2)",
            "\n     are really categorical. ")
      cat("\n Finding ", q[idx2],
          " unique values for categorical covariates in cols",
          p0+p1+1,"--", p0+p1+p2,"\n")
      if (is.null(cov.pi2)){
        cov.pi2 <- vector("list",p2)
        for(j in 1:p2)
          cov.pi2[[j]] <- rep(0.1,q2[j])
      }
      if (length(unlist(cov.pi2)) != sum(q2)){
        cat("\n *** Error: need length(pi2[j]) = # levels of j-th ",
            "\n     categorical covariate.\n")
        return(-1)
      }
    }
    ## initial clustering
    if(is.null(k0)){
      k0 <- ifelse(p2>0,0,n)
      cat(" setting default for k0=",k0,"\n")
    }
    if (!is.element(k0,c(0,1,n))){
      cat("\n *** Error: need k0= 0, 1 or n. \n")
      return(-1)
    }
    if (k0 == 0){ # initialize by cross tabulation of categoricals
      Y2 <- Y[,idx2]
      q2 <- q[idx2]
      member0 <- cross.tab(Y2,q2)
      k0 <- length(unique(member0))
      cat("\n Using cross-tabulation by categoricals as member0.\n")
    } else
      member0 <- 0 # dummy
    ## response variable
    if (!is.element(censoring,c(0,1))){
      cat("\n *** Error: need censoring=0 or 1.\n")
      return(-1)
    }
    if ((p0>1) & (censoring==1)){
      cat("\n *** Error: censoring=1 only with p0=1 response.\n")
      return(-1)
    }
    idx0 <- 1:p0
    Y0 <- as.matrix(Y[,idx0]) # need as.matrix for p0=1
    mhat <- apply(Y0,2,mean)
    V <- var(Y0)
    if (p0>1)
      v <- diag( diag(V) )   # as.matrix in case p0=1
    else # p0==1
      v <- as.matrix(V)      ## in old version of R var returns a scalar for p=1
    if(is.null(S.init)){
      S.init <- 0.25*V
      cat(" setting default for S.init = 1/4*var(Y).\n")
    }
    if(is.null(R)){
      R <- 0.25*v
      cat(" setting default for R = 1/4*diag(v).\n")
    }
    if(is.null(m.init)){
      m.init <- mhat
      cat(" setting default for m.init = mean(Y).\n")
    }
    if(is.null(A.inv)){
      A.inv <- (1/v)
      cat(" setting default for A = diag(v).\n")
    }
    if(is.null(a)){
      a <- mhat
      cat(" setting default for a = mean(Y).\n")
    }
    ## py-init
    if (!is.element(py,c(0,1))){
      cat("\n *** Error: need py=0 or 1.\n")
      return(-1)
    }
    if(py==1){
      if(is.null(iy)){
        iy <- 0
        cat("\n Computing marginals for response in column",iy+1,"\n")
      }
      if (iy >= p0){
        cat("\n *** Error: iy needs to indicate one of the first p0=",p0,
            "\n     columns (CAREFUL: iy starts counting at 0 for col. 1.\n")
        return(-1)
      }
      yy <- Y[,iy+1]
      if(is.null(ygrid)){
        ygrid <- range(yy)
      }
      if((ygrid[1]<min(yy)) | (ygrid[2]>max(yy))){
        cat("\n *** Error: can not set range of ygrid beyond range of data.\n")
        return(-1)
      }
      if(is.null(nx)){
        if(!is.null(xgrid))
          nx <- nrow(xgrid)
        else{
          if(p1+p2==0){
            cat("\n Using first 5 cov vectors as xgrid.\n")
            xgrid <- Y[1:5,-(1:p0)] # simply use first 5 design vectors
            nx <- 5
          } else { # use cross tab of all categoricals & (hi,med,lo) cont
            cat("\n Using cross tabulation of categoricals for xgrid.\n")
            p12 <- p1+p2
            q12 <- c(rep(3,p1),q2)
            nx <- prod(q12)
            xgrid <- cross.tab2(q12,nx,p1)
            if (p4>0){
              m4 <- round( apply(Y[,(p-p4+1):p4],2,mean) )
              for(j in 1:p4)
                xgrid <- cbind(xgrid,m4[j])
            }
          } # xgrid
        } # is.null xgrid
      }
    } # py==1

    ## pyvec-init
    if (!is.element(pyvec,c(0,1))){
      cat("\n *** Error: need pyvec=0 or 1.\n")
      return(-1)
    }
    if(pyvec==1){
      if(is.null(nx)){
        if(!is.null(xgrid))
          nx <- nrow(xgrid)
        else{
          if(p1+p2==0){
            cat("\n Using first 5 cov vectors as xgrid.\n")
            xgrid <- Y[1:5,-(1:p0)] # simply use first 5 design vectors
            nx <- 5
          } else { # use cross tab of all categoricals & (hi,med,lo) cont
            cat("\n Using cross tabulation of categoricals for xgrid.\n")
            p12 <- p1+p2
            q12 <- c(rep(3,p1),q2)
            nx <- prod(q12)
            xgrid <- cross.tab2(q12,nx,p1)
            if (p4>0){
              m4 <- round( apply(Y[,(p-p4+1):p4],2,mean) )
              for(j in 1:p4)
                xgrid <- cbind(xgrid,m4[j])
            }
          } # xgrid
        } # is.null xgrid
      }
    } # pyvec==1

    ## MCMC tuning pars
    if (n.discard >= n.iter){
        cat("\n *** Error: n.iter <= n.discard. \n")
        return(-1)
      }
    if (n.batch >= n.iter/2){
      cat("\n *** Error: n.iter/2 <= n.batch. \n")
      return(-1)
    }
    ## warnings
    if (cc < p0+2)
      cat(" *** Warning: should use c > p+2 for good mixing MCMC.\n")
    if (qq < p0+2)
      cat(" *** Warning: should use q > p+2 for good mixing MCMC.\n")
    if (s < p0+2)
      cat(" *** Warning: should use s > p+2 for good mixing MCMC.\n")
    
    ## 3. write init file
    options(digits=2)
    out <- file("init.mdp",open="w")
    cat(" n ",n,
        "\n p",   p,
        "\n p1",  p1,
        "\n p2",  p2,
         file=out)
      
    if (p1>0)
      cat(
        "\n cov-Vj1",   format(cov.Vj1),
        "\n cov-mean1", format(cov.mean1),
        "\n cov-B1",    format(cov.B1),
        "\n cov-alpha1", rep(1,p1),     # dummy
          file=out)
    if (p2>0){
      
    }
    if (p4>0){
      cat("cov-a4 ", cov.a4, 
          "cov-b4 ", cov.b4, 
          "cov-alpha4 ", rep(1,p4),
          file=out)
    }
    cat(
        "\n niter ", n.iter,
        "\n m_prior ", m.prior, "B_prior ", B.prior,
        "\n S_prior ", S.prior,
        "ndiscard ", n.discard, "nreinit ", n.reinit,
        "npi ", n.batch, "\n",
        "\n npredupdate ", n.predupdate,
        "\n nprintallpars ", n.printallpars,
        "\n verbose ", verbose,
        "\n py ", py,
        "\n pyvec ", pyvec,
        "\n s ", s,
        "\n S-init \n", format(S.init),
        "\n q ", qq, file=out)
    if (p0>1)
      cat(
        "\n R-inv  \n", format(solve(R)),
        "\n B-scale \n", format(B.scale),
        "\n c ", cc,
        "\n m-init ", format(m.init),
        "\n aa ", format(a),
        "\n A-inv \n", format(A.inv),
          file=out)
    else
      cat(
        "\n R  \n", format(R),
        "\n B-scale \n", format(B.scale),
        "\n c ", cc,
        "\n m-init ", format(m.init),
        "\n aa ", format(a),
        "\n A \n", format(1/A.inv),
          file=out)
          
          bal=bal+.001
          bal=as.double(bal)
          ab=ab+.001
          ab=as.double(ab)
          cd=cd +.001
          cd =as.double(cd)

    cat("\n alpha-init ", alpha,
        "\n a0 ", a0, " b0 ", b0,
        "\n k0 ", k0, 
          "\n m0 ", bal, " ab ", ab, " cd ", cd,
        "\n member0 ", member0, "\n", # dummy if k0=1 or n
        file=out)
    close(out)

    ## 4. write data file
    out <- file("data.mdp",open="w")
    write(format(t(Y)),ncolumns=p,file=out)
    close(out)
    ## 4. write mis file
    out <- file("mis.mdp",open="w")
    write(format(t(mis)),ncolumns=p2,file=out)
    close(out)


    ## 5. write py-init file
    if (py==1){
      out <- file("py-init.mdp",open="w")
      cat("\n ny ",ny, "nx ",nx, "yi ", iy,
          "\n ygrid ", ygrid,
          "\n xgrid \n",
          file=out)
      write(format(t(xgrid)),ncolumns=ncol(xgrid),file=out)
      close(out)
    }

    ## 5b. write pyvec-init file
    if (pyvec==1){
      out <- file("pyvec-init.mdp",open="w")
      cat("\n nx ",nx, 
          "\n xgrid \n",
          file=out)
      write(format(t(xgrid)),ncolumns=ncol(xgrid),file=out)
      close(out)
    }
    
    ## 4. call C executable
    info <- .C("ppmx",package="ppmx")
    ## 5. return error code
    return(info)
  }


## auxilary functions
n.unique <- function(x)
  { # returns number of unique values
    length(unique(x))
  }

cross.tab <-  function(Y2,q2)
{ ## computes cross tabulation of categorical var's in the columns of Y2,
  ## into k0 distinct combinations x*[j], j=0..k0-1, and
  ## returns vector of indicators s0[i]=j iff X[i]=x*[j]
  ## q2[j]=number of distinct values of j-th categorical var
  p2 <- length(q2)
  
  Y2 <- t( t(Y2) - apply(Y2,2,min)) # now each categorical starts at 0
  s0 <- Y2[,1]
  offset <- q2[1]
  for(j in 2:p2){
    s0 <- s0+Y2[,j]*offset
    offset <- q2[j]*offset
  }
  return(s0)
}

cross.tab2 <- function(q12,nx,p1)
  { ## returns list of nx design vectors
    ## with all possible cominations of the p12 variables with
    ## q12[j]= n levels of j-th variable
    ## (to be used with q12[j]=3 for continous covariates to return
    ## -1,0,1 for lo, med, hi

    p12 <- length(q12)
    xgrid <- matrix(0,nrow=nx,ncol=p12)
    for(i in 1:nx){
      x <- i-1
      for(j in 1:p12){
        xgrid[i,j] <- x %% q12[j]
        x <-  x %/% q12[j]
      }
    }
    if(p1>0)
      xgrid[,1:p1] <- xgrid[,1:p1]-1 # to code -1,0,1
    return(xgrid)
  }

