/* ============================================================= 
   clustering w. covariates
   
   /* =============================================================  */


#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include "ppmx.h"
#include "bayes.h"
#include "rand.h"
#include "nrutil.h"
#include "matrix.h"
#include "vector.h"
#include "regr.h"
#include "rand-nr.h"
#include "mess.h"
#include "interface.h"
#include "predict.h"


#define Ndiscard 50          /* number of discarded iterations */

/* ***********************************************************
   all MDP pars
/* *********************************************************** */
struct MDP_PARS mdp;
struct SIM_PAR sim;
struct DTA dta;

int n,p,p0,p1,p2,p3,p4;

double *m_aux, *s_aux, *q_aux, /* auxiliary arrays of dim nobs
			   for temporary results */
  *c_aux,*l_aux,*pr_aux;


void copy(double *X, double *Y, int nrow)
{
    register int i;
    
	
    for(i = 0; i < nrow; i++) {
                   X[i] = Y[i];
        }
    
}




/* -----------------------------------------------------------
   Gibbs iterations
/* ----------------------------------------------------------- */
int n_iter,             /* max iterations */
  iter,                 /* current iter number */
  n_discard,            /* discard the first n_discard draws */
  n_reinitialize,         /* reinitialize configurations */
  n_predupdate,
  n_pi;                 /* every n_pi update expectations etc. */

int n_printallpars;     /* intervals to print all pars if 
			   verbose > 2 */
int m_prior, B_prior, S_prior; 
int n_class0, *member0; /* initial config */

/* -----------------------------------------------------------
   various output files
/* ----------------------------------------------------------- */
FILE *parFile, *muFile, *VFile, *SFile, *meanFile, *BFile, *memberFile,
  *ifile,
  *mj1file, *pij2file, *aj4file;

/* -----------------------------------------------------------
   debugging
/* ----------------------------------------------------------- */
int dbg=0;

/* ============================================================= 
   main
/* ============================================================= */ 
// main()
void ppmx()
{ 
  init(); 
  init_output();
  pred_init(mdp,dta);
  init_config();
  gibbs(); 
  finish();
}				  

/* ============================================================= 
   functions called from main
/* ============================================================= */ 
/* ***********************************************************
   init
/* ***********************************************************
   reads in data, initial par values */
void  init()
{
  int i, j;
  double r, B_scale, y, cy, d, R1, A1;
  char line[250];

/* -----------------------------------------------------------
   read in sample size and dim
/* ----------------------------------------------------------- */
  openIn("init.mdp");
  scanInt("n", &dta.n_obs);
  n = dta.n_obs;
  messinttab("nobs", n);
  scanInt("p", &p);    /* dim of obs vector, INCL covariate vector
			    for clustering */
  /*scanInt(" p0 ", &p0);  /* ... WITHOUT covariate for clustering */
	
  messint("p ",p);
  dta.p = p;
  dta.p0 = 1; 
p0=1;
  /*scanInt(" censoring ", &mdp.cens);*/
	mdp.cens=0;
  if (mdp.cens==1) // censoring, need p>=3 for Y,C,delta
    if (p0 != 1)
      err_msg("init","need p0=1 with censoring",0,0,0);

/* -----------------------------------------------------------
   allocate memory for pars & data
/* -----------------------------------------------------------  */
Rprintf("ok1");
  dta.Y = dmatrix(0,n-1,0,p-1);
  dta.CY = dvector(0,n-1); // censoring times (if cens=1)
  dta.delta = ivector(0,n-1); // censoring indicator
  mdp.V = darray_3(0,n);
Rprintf("ok2");
  if (p0>1){
    mdp.Vt = darray_3(0,n);
    mdp.Vj = darray_3(0,n);
    mdp.V_cd_inv = darray_3(0,n);
    mdp.Vt_cd_inv = darray_3(0,n);
    for(j=0;j<=n;j++){             /* don't forget j=n for new */
      mdp.V[j] = dmatrix(0,p0-1,0,p0-1);
      mdp.Vj[j] = dmatrix(0,p0-1,0,p0-1);
      mdp.V_cd_inv[j] = dmatrix(0,p0-1,0,p0-1);
      mdp.Vt[j] = dmatrix(0,p0-1,0,p0-1);
      mdp.Vt_cd_inv[j] = dmatrix(0,p0-1,0,p0-1);
    }
    mdp.mu = dmatrix(0,n,0,p0-1);
    mdp.mj = dmatrix(0,n,0,p0-1);
	 	  
    mdp.mean = dvector(0,p0-1);

    mdp.S = dmatrix(0,p0-1,0,p0-1);
    mdp.S_cd_inv = dmatrix(0,p0-1,0,p0-1);
    mdp.S_inv = dmatrix(0,p0-1,0,p0-1);
    mdp.R_inv = dmatrix(0,p0-1,0,p0-1);
    mdp.B = dmatrix(0,p0-1,0,p0-1);
    mdp.B_inv = dmatrix(0,p0-1,0,p0-1);
    mdp.B_cd_inv = dmatrix(0,p0-1,0,p0-1);
    mdp.C = dmatrix(0,p0-1,0,p0-1);

    mdp.aa = dvector(0,p0-1);
    mdp.A_inv = dmatrix(0,p0-1,0,p0-1);
  } else { // p0=1
    mdp.mu1 = dvector(0,n);
    mdp.V1 = dvector(0,n);
    mdp.Vt1 = dvector(0,n);
    mdp.mj1 = dvector(0,n);
    mdp.Vj1 = dvector(0,n);
  }
   
  /* note: count, first & upd allow for one extra class =
     "new class" */
  mdp.count = ivector(0,n);
  mdp.first = ivector(0,n);
	mdp.mjj=dvector(0,n-1);
	mdp.mll=dvector(0,n-1);
	mdp.sig=dvector(1,n-1);

  mdp.member = ivector(0,n-1);
  mdp.updated = ivector(0,n);
  mdp.prev = ivector(0,n);
  mdp.next = ivector(0,n);
  mdp.new = ivector(0,n);

  member0 = ivector(0,n-1);

  m_aux = dvector(0,n-1);
  s_aux = dvector(0,n-1);
  q_aux = dvector(0,n-1);
  c_aux = dvector(0,n-1);
  l_aux = dvector(0,n-1);
  pr_aux = dvector(0,n-1);

/* -----------------------------------------------------------
   memory alloc for covariates
/* ----------------------------------------------------------- */
  scanInt("p1", &dta.p1); // n continuous cov's
  scanInt("p2", &dta.p2); // n categorical cov's
  /*dta.p1=0;
    dta.p2=7;*/
  dta.q2 = ivector(0,dta.p2); // n levels for each categorical
  /*scanIntArray(" q2 ", dta.q2, dta.p2);*/
	int ii;
	for(ii=0;ii<dta.p2;ii++){
		if(ii< 9) dta.q2[ii]=2;
if(ii >=9) dta.q2[ii]=15;
	}
  dta.mx_q2=i_max(dta.q2,&i,dta.p2); // remember the max n levels
  /*scanInt(" p3 ", &dta.p3); // n ordinal cov's
  scanInt("p4", &dta.p4); // n count cov's */
	dta.p3=0;
	dta.p4=0;
  p1=dta.p1;
  p2=dta.p2;
  p3=dta.p3;
  p4=dta.p4;
Rprintf("ok-n");

dta.mis=dmatrix(0,n-1,0,p2-1);
  if (dta.p3>0)
    error("init","ordinal cove not yet implemented",0);
  if (dta.p0+dta.p1+dta.p2+dta.p3+dta.p4 != dta.p)
    error("init","p0+p1+p2+p3+p4 != p, p=",p);
  sim.V1=   dvector(0,p1);
  sim.mean1=dvector(0,p1);
  sim.B1=   dvector(0,p1);
  sim.mj1=  dmatrix(0,n-1,0,p1);
  sim.sj1=  dmatrix(0,n-1,0,p1);
  sim.pi2=  dmatrix(0,p2,0,dta.mx_q2);  // prior Dir par's
  sim.pij2= darray_3(0, n);
  for(j=0;j<n;j++)             /* don't forget j=n for new */
    sim.pij2[j] = dmatrix(0,p2-1,0,dta.mx_q2);
  /* include extra q2 element to be used for normalization const */
  sim.aj4 = dmatrix(0,n-1,0,p4);
  sim.bj4 = dmatrix(0,n-1,0,p4);
  sim.a4 = dvector(0,p4);
  sim.b4 = dvector(0,p4);
  sim.alpha1=dvector(0,p1);
  sim.alpha2=dvector(0,p2);
  sim.alpha4=dvector(0,p4);
	sim.dahl=0;
  /*scanInt(" cov-dahl ", &sim.dahl); /* indicator for using David
				       Dahl's method */
  if (p1>0){
    scanDoubleArray("cov-Vj1",sim.V1,p1);
    scanDoubleArray("cov-mean1",sim.mean1,p1);
    scanDoubleArray("cov-B1",sim.B1,p1);
    scanDoubleArray("cov-alpha1",sim.alpha1,p1);
  }

int gf;
for(j=0;j<p2;j++){
  if(j< 9) sim.pi2[j][0]=.1; sim.pi2[j][1]=.1;
if(j>=9){
for(gf=0;gf<15;gf++) {
sim.pi2[j][gf]=.1;
}

}
}

   if (p2>0)
{
for(j=0;j<p2;j++){
sim.alpha2[j]=1;
}
}
   /* scanDoubleArray(" cov-alpha2 ",sim.alpha2,p2);*/

  if (p4>0){
    scanDoubleArray("cov-a4",sim.a4,p4);
    scanDoubleArray("cov-b4",sim.b4,p4);
    scanDoubleArray("cov-alpha4",sim.alpha4,p4);
  }
Rprintf("p0 is %d", p0);
Rprintf("ok-m");
/* -----------------------------------------------------------
   read in hyperpars and initial values
/* ----------------------------------------------------------- */

  scanInt("niter",&n_iter);
  scanInt("m_prior", &m_prior);
  scanInt("B_prior", &B_prior);
  scanInt("S_prior", &S_prior);
  scanInt("ndiscard",&n_discard);
  scanInt("nreinit",&n_reinitialize);
  scanInt("npi",&n_pi);
  scanInt("npredupdate",&n_predupdate);
  scanInt("nprintallpars",&n_printallpars);
  scanInt("verbose", &mdp.verbose);
  scanInt("py", &mdp.py);
  scanInt("pyvec", &mdp.pyvec);
  scanInt("s", &mdp.s);

  if (p0>1){
    /* read and invert S */
    scanDoubleMatrix(" S-init ",mdp.S,p0,p0);
    d_invert(p0,mdp.S,mdp.S_inv);
    d_chol_decomp(p0,mdp.S_inv,mdp.S_cd_inv);

    scanInt(" q ",&mdp.q);
    scanDoubleMatrix(" R-inv ",mdp.R_inv, p0,p0);

    /* read and invert B */
    scanDouble(" B-scale ", &B_scale);
    rA(B_scale,mdp.S,mdp.B,p0,p0);   // B = B_scale*S
    d_invert(p0,mdp.B,mdp.B_inv);
    d_chol_decomp(p0,mdp.B_inv,mdp.B_cd_inv);

    scanInt(" c ",&mdp.c);
    // scanDoubleMatrix(" C ",mdp.C, p,p); replaced by the following 2 lines
    A_zero(mdp.C,p0);   // initialize
    d_invert(p0,mdp.R_inv,mdp.C);
    rA(B_scale,mdp.C,mdp.C,p0,p0);
    scanDoubleArray(" m-init ",mdp.mean, p0);
    scanDoubleArray(" aa ", mdp.aa, p0);
    scanDoubleMatrix(" A-inv ", mdp.A_inv, p0,p0);
  } else { // p0=1
    scanDouble("S-init", &mdp.S1);
    scanInt("q", &mdp.q);
    scanDouble("R", &R1);
    mdp.R1_inv = 1.0/R1;
    scanDouble("B-scale", &B_scale);
    mdp.B1 = B_scale*mdp.S1;
    scanInt("c",&mdp.c);
    mdp.C1 = B_scale*1.0/mdp.R1_inv;
    scanDouble("m-init",&mdp.mean1);
    scanDouble("aa",    &mdp.aa1);
    scanDouble("A", &A1);
    mdp.A1_inv = 1.0/A1;
  }

  scanDouble("alpha-init",&mdp.alpha);
  scanDouble("a0", &mdp.a0);
  scanDouble("b0", &mdp.b0);
  
Rprintf("2p0 is %d",p0);
/* -----------------------------------------------------------
   initial config
/* ----------------------------------------------------------- */
  scanInt("k0", &n_class0);
scanDouble("m0", &mdp.bal);
scanDouble("ab", &mdp.ab);
scanDouble("cd", &mdp.cd);



  if (n_class0 == 1)
    for(i=0;i<n;i++)
      member0[i] = 0;
  else if (n_class0 == n)
    for(i=0;i<n;i++)
      member0[i] = i;
  else if (n_class0 == -2){
    for(i=0;i<n;i++)
      member0[i] = i%2;
    n_class0 = 2;
  }
  else
    {Rprintf("ncc"); /*scanIntArray(" member0 ", member0, n); */}

  closeIn();

/* -----------------------------------------------------------
   read in data
/* ----------------------------------------------------------- */

  ifile = openIn("data.mdp");
  if (!mdp.cens)
    scanDoubleMatrix("Y",dta.Y, n, p);
  else { // 2nd and 3rd column are censoring indicator and time
    for(i=0;i<n;i++){
      fscanf(ifile, " %f %lg %lg ", &y,&d,&cy);
      if (d-floor(d) != 0) 
	err_msg("init","non-integer value for censoring indicator, line %d",
		i,0,0);
      dta.Y[i][0]=y; dta.delta[i]=d; dta.CY[i]=cy;
      if ( (dta.delta[i]!=0) & (dta.delta[i]!=1))
	err_msg("init","delta[%d]=%d. Need 0 or 1.", i,dta.delta[i],0);
      scanDoubleArray(" ",&dta.Y[i][1],p-1); // cov's
    }
  } // censoring
Rprintf("3p0 is %d", p0);
  closeIn();
  
ifile = openIn("mis.mdp");
    scanDoubleMatrix("mis",dta.mis, n, p2);
closeIn();

  if (sim.dahl==1)
    dahl_init();
}

void dahl_init()
{
  int i1,i2,first=1;
  double dij;

  for(i1=0;i1<n;i1++){
    for(i2=0;i2<i1;i2++){
      dij = dahl_dist(dta.Y[i1], dta.Y[i2]);
      if (first==1)
	sim.dstar=dij;
      else if (dij > sim.dstar)
	sim.dstar = dij;
      first = 0;
    }
  }
  sim.dstar = sim.dstar*1.01; /* to avoid dist=0 later.. */
}

double dahl_dist(double *x1, double *x2)
{
  double dij;
  int j;

  dij=0;
  for(j=0;j<p1;j++) //continuous covariates
    dij += sqr(x1[p0+j]-x2[p0+j]);
  for(j=0;j<dta.p2;j++) // categorical covariates
    if (x1[p0+p1+j] != x2[p0+p1+j])
      dij += 10.0; 
  for(j=0;j<dta.p4;j++) // count covariates
    dij += sqr(x1[p0+p1+p2+p3+j]-
	       x2[p0+p1+p2+p3+j]);
  return(dij);
}

double dahl_hil(double *xi, int k)
{ // log \sum (h*-hij) 
  int i;
  double hl, lghl;
  
  if (mdp.count[k]==0)
    lghl=0;
  else{
    hl=0;
    for(i=mdp.first[k];i!=-1;i=mdp.next[i])
      hl += sqr( sqr(sim.dstar - dahl_dist(xi, dta.Y[i]) ) );
    lghl = log(hl);
  }
  return(lghl);
}

  
/* ***********************************************************
   open output files
/* *********************************************************** */
void init_output()
{
  parFile = openOut("par.mdp");
  muFile = openOut("mu.mdp");
  VFile = openOut("V.mdp");
  SFile = openOut("S.mdp");
  memberFile = openOut("member.mdp");  /* configs */
  BFile = openOut("B.mdp");
  meanFile = openOut("mean.mdp");

  mj1file = openOut("mj1.mdp");
  pij2file = openOut("pij2.mdp");
  aj4file = openOut("aj4.mdp");
  fclose(mj1file); 
  fclose(pij2file); 
  fclose(aj4file); 

  fclose(muFile); 
  fclose(SFile); 
  fclose(meanFile); 
  fclose(VFile);
  fclose(BFile);
  fclose(parFile);
}

/* ***********************************************************
   initial configuration
/* *********************************************************** */
void init_config()
{
	Rprintf("1a");
  int i, k, ct;

  /* compute inverse and cd of S */
  if (p0>1){
    d_invert(p0,mdp.S,mdp.S_inv);
    d_chol_decomp(p0,mdp.S_inv,mdp.S_cd_inv);
  }
  
  /* create n empty groups */
  mdp.n_class = 0;
Rprintf("1c");
  for (k=0;k<n;k++){
    mdp.count[k] = 0;
    mdp.first[k] = -1;
    mdp.updated[k] = 0;
    mdp.new[k] = 0;
    sample_V(k);
  }

  /* put obs's in groups */
  for (i = 0; i < n; i++){
    k = member0[i];
    add_class(i,k);
  }

  /* sample mu's */
Rprintf("1b");
  for(k=0;k<mdp.n_class;k++){  // was k<n
    sample_mu(k);
  }
  check_class();
	Rprintf("1a");
}

/* *************************************************
   reinitialize_config
 ************************************************* */
void reinitialize_config()
{
  int i;

  /* remove old config */
  for (i=0;i<n;i++)
    take_out(i);
  init_config();
}



void setup_mjj(int k){
	/* linear */
	double *yy; double *mjj; double **x; int kk; double *res; int *nn;
	double *beta; double *offset;
	yy=dvector(0,mdp.count[k]-1);
	mjj=dvector(0,mdp.count[k]-1);
	res=dvector(0,mdp.count[k]-1);
	offset=dvector(0,n-1);
	double *sig;
	sig=dvector(0,n-1);

	int ii,jj;
	x=dmatrix(0, mdp.count[k]-1, 0,  (p1+p2));
	int lk=0;
	for(ii=0;ii<=(p1); ii++){
		/*if(mdp.cvv[k][ii]!=0){*/
		kk=0;
		for(jj=mdp.first[k]; jj!=-1; jj=mdp.next[jj]){
			if(ii==0) x[kk][ii]=1;
			if(ii>0) x[kk][ii]=dta.Y[jj][ii];
			kk=kk+1;
			
		}
		
		/*}*/
		lk=lk+1;
	}
	
	
	
	
	kk=0;
	for(int jj=mdp.first[k]; jj!= -1;  jj=mdp.next[jj]  ){
		yy[kk]=dta.Y[jj][0];
		kk=kk+1;
	}
	
	
	/*regr2(mjj, yy,  x, mdp.count[k], (p1+1),
		  0, sig);
	
	int oi=0;
	for(int jj=mdp.first[k]; jj!= -1;  jj=mdp.next[jj]  ){
		mdp.mjj[jj]=mjj[oi];
		oi=oi+1;
	}
	mdp.sig[k]=.0625;
	Rprintf("\t %f",mdp.sig[k]);*/
	/*logistic*/
	int py=p1+p2;
	beta=dvector(0,(py));
	/*nn=ive(n,2);
	/*int oi=0;
	for(oi=0;oi<n;oi++){
		nn[oi]=2;
	}*/
	int px=p1+p2+1;
	/*Rprintf("ok1");*/
	bayes_regr_logistic2(res, beta, yy, x, offset, mdp.count[k],px, sig, 315, 0);
	/*Rprintf("ok2");*/
	 int oi=0;
	for(int jj=mdp.first[k]; jj!= -1;  jj=mdp.next[jj]  ){
		mdp.mll[jj]=res[oi];
		oi=oi+1;
	}

}




/* ***********************************************************
   Gibbs Sampler
/* *********************************************************** */
void gibbs()
{
  int time, last_init, k,ii; 

  last_init = 0;
  for (iter = 0; iter < n_iter; iter++){
    if ( (iter % n_reinitialize == 0) & (iter > 0)){
      reinitialize_config();
      last_init = iter;
    }
    if (mdp.verbose > -1) print_pars(iter);
    if (S_prior) sample_S();
    mdp.kmax = mdp.n_class;

    sample_config();
	  Rprintf("ok 3");

    for(k=0; k<mdp.n_class; k++){
      sample_mu(k);
		if(iter>= 11539) {if(mdp.count[k]>4) setup_mjj(k);}  /*if(iter>= 11539){ Rprintf("%d \t %d \t",mdp.count[k],k);}*/
      sample_V(k);
    } 

    /*sample_alpha(); */ 
    if (B_prior) sample_B();
    if (m_prior) sample_m();
    if (iter-last_init > n_discard){
      update_nclass();
      if (iter % n_pi == 0){
	print_pi();
	write_pars(iter);
	write_nclass();
      }
      if (iter % n_predupdate == 0) /* update predictive */
	pred_update(iter, mdp,dta, sim);
    }
  }
	FILE *pp;
	pp=openOut("pp0.mdp");
	for(ii=0;ii<n;ii++){
		fprintf(pp, "%f \n", mdp.mll[ii]);	
	}
	fclose(pp);
	
}

/* ***********************************************************
   finish
/* *********************************************************** */
void finish()
{
     print_pi();
     pred_finish(mdp);
}		   
  
/* ============================================================= 
   Conditional draws
/* ============================================================= */ 
	  
/* ***********************************************************
   sample config
/* ***********************************************************
   does the Gibbs over configurations
   Pr(y[i] in class j | ...) proportional to
   for j=0,..n_class-1:
      count[j]*MV-N(y[i]; mut[j], Vt[j])
      mut[j] and Vt[j] computed in make_mut_Vt
   for j=n_class (i.e. new class): same expression with 
                                   count[n_class] = alpha
*/

void sample_config()
{
	int l, k_new, ind;
	double u, mx, hC, mxh, lgr;   /* prob of pi[ind] = pi*[k] */ 
	int si, j;
	
	/* allocate memory */
	
	/*mdp.alpha=1;*/
Rprintf("alok");
	for (ind = 0; ind < n; ind++){
		u = runif();
		if (mdp.count[mdp.member[ind]] == 1)
		/* toss coin with prob 1/k */
			if (u > 1.0/(1.0*mdp.n_class)) continue;
		if (mdp.member[ind]==-1)
			err_msg("sample_config","clusterless point %d!",ind,0,0);
		take_out(ind);
/*Rprintf("fine");*/
		
		/* compute prob pi[ind] = pi*[k]       */
/*if(ind==351) Rprintf("class is %d",mdp.n_class);*/
		for(l=0;l<mdp.n_class;l++){
/*if(mdp.count[l]==0) Rprintf("indd %d \t %d",ind, l);*/
			if (mdp.updated[l]==0)
				setup_mu(l,0);
			l_aux[l] = loglik(ind,l,&m_aux[l],&s_aux[l]); // save in aux
			// arrays for debug
			/*l_aux[l]=0;*/
			
			c_aux[l] = logpr(dta.Y[ind],ind,l); 
			pr_aux[l] = log(1.0*mdp.count[l]);
			q_aux[l] =  pr_aux[l] + c_aux[l] + l_aux[l];
                /* if(ind==351) Rprintf("\n ll is %f \t %f \t %f,",l_aux[l],pr_aux[l],c_aux[l]);*/

		}	
		
		if (sim.dahl==1){ /* normalize to sum prior prob = n-1 */
			mxh = x_max(c_aux, &l, mdp.n_class); /* for scaling */
			for(l=0,hC=0.0; l<mdp.n_class; l++)
				hC += exp(c_aux[l]-mxh);
			/* normalization const for Dahl's  model */
			lgr = log( (n-1.0)/hC ); /* multiply with r to assure sum pr*q = n-1 */
			for(l=0;l<mdp.n_class;l++){
				q_aux[l] = c_aux[l] - mxh /*scaling*/ + lgr /* to stdze */ + l_aux[l];
				c_aux[l] = c_aux[l] - mxh /*scaling*/ + lgr /* to stdze */;
				/* debugging */
			}
			/* note: replacing pr_aux entirely! */
		}
		
		
		/* Prob of new class */
		/* q_aux[n_class = alpha * MV-N(y[i],m,V+tau*B)*/
		if (mdp.updated[mdp.n_class]==0)
			setup_mu(mdp.n_class,0);
		l = mdp.n_class;
	l_aux[l] =  loglik(ind,l,&m_aux[l],&s_aux[l]);
		/*l_aux[l]=0;*/

		c_aux[l] =  logpr(dta.Y[ind],ind,l); 
		pr_aux[l] = log(mdp.alpha/(l+1.0)); /* 0 for dahl */
		q_aux[l] =  pr_aux[l] + c_aux[l]+ l_aux[l];
		 /*if(ind==351) Rprintf("\n lln is %f \t %f \t %f,",l_aux[l],pr_aux[l],c_aux[l]);*/
		/* draw the index of the new class for ind */
		mx = x_max(q_aux,&l,mdp.n_class+1);  /* standardize probs */
		for(l=0;l <= mdp.n_class;l++)
			q_aux[l]= exp(q_aux[l]-mx);
		/* draw the index of the new class for ind */
		multinomial(1,mdp.n_class+1,q_aux,&k_new);   
/*if(ind==351) Rprintf("knew is %d", k_new); */ 
		if (mdp.cens==1)
			if (dta.delta[ind]==0){ /* censored */
				dta.Y[ind][0] = 
				norm_trunc_left(m_aux[k_new],s_aux[k_new],dta.CY[ind]);
				if (s_aux[k_new]<0.00001)
					err_msg("sample_config","small sd for ind=%d, k=%d.",ind,k_new,0);
			}
		add_class(ind,k_new);
		/*Rprintf("well %d", ind);*/
		if (k_new == mdp.n_class){ 
			if (k_new > mdp.kmax) mdp.kmax = k_new;
		}
	}
	
}


double logpr(double *x, int i, int l){
  /* prior prob for l-th class 
     index i is only used for error messages 
     NOTE: i is meaningless when called from predict!! */
  int j, yj, q;
  double sumpij, aj,bj, lpr,yjd, sd;

  if (sim.dahl==1)
    lpr = dahl_hil(x,l); /* returns 0 for l==k+1 */
  else{
    if (l > mdp.n_class)
      err_msg("logpr","l=%d > nclass=%d.",l,mdp.n_class,0);

    lpr=0;
    for(j=0;j<dta.p1;j++){  // continous covariates
      sd = sqrt(sim.sj1[l][j]+sim.V1[j]);
      lpr += d_normal_dens(x[p0+j],sim.mj1[l][j],sd)*
	sim.alpha1[j];
    }
    for(j=0;j<dta.p2;j++){  // categorical covariates
      q = dta.q2[j];
      sumpij = sim.pij2[l][j][q];
      yjd = x[p0+p1+j];
      if ( yjd-floor(yjd) != 0)
	err_msg("sample_config","y[%d][%d] not integer", 
		i,p0+p1+j,0);
      yj=yjd;
      if ( (yj>=q) | (yj<0)) 
	err_msg("sample_config","y[%d][%d]=%d out of range",
		i,j,yj);
     if(dta.mis[i][j]==1){ lpr += ( log(sim.pij2[l][j][yj]+.01) - log(sumpij+.01) )*
	sim.alpha2[j];}
    }
    for(j=0;j<dta.p4;j++){ // count covariates
      yjd = x[p0+p1+p2+p3+j];
      if ( yjd-floor(yjd) != 0)
	err_msg("sample_config","y[%d][%d] not integer", 
		i,p0+p1+p2+p3+j,0);
      aj = sim.aj4[l][j];
      bj = sim.bj4[l][j];
      lpr += ( aj*log(bj)-(aj+yjd)*log(bj+1.0)+lgamma(aj+yjd)-lgamma(aj) )*
	sim.alpha4[j];
    }
  }
  return(lpr);
}

double loglik(int i,  int l, double *m, double *s){
  /* compute likelihood for i-th observation with moments (mj,Vj) */
  double lp,F; 
	int it,jt;
	int j,k,offset; double sig;
	  if (l > mdp.n_class)
    err_msg("loglik","l=%d > nclass=%d.",l,mdp.n_class,0);

  if (!mdp.cens){
   
		if(iter <= 11539) {lp = d_normal_dens(dta.Y[i][0],mdp.mj1[l],sqrt(mdp.Vt1[l]));
			/*lp=log(1.0*dta.Y[i][0]*(mdp.mj1[l]) + 1.0*(1-dta.Y[i][0])*(1-mdp.mj1[l]));*/
			/*if(iter>5000) { if(i==0){ Rprintf(" mj1l is %f",mdp.mj1[l]);}}*/
		
		}
	  if(iter>11539)	{
	lp = d_normal_dens(dta.Y[i][0], mdp.mjj[i], sqrt(mdp.sig[l]));
		 /* lp=log(exp(dta.Y[i][0]*mdp.mll[i])/(1+ exp(mdp.mll[i]) ));*/
	  }
  }
	else{
    *m = mdp.mj1[l];
    *s = sqrt(mdp.Vt1[l]);
    if (dta.delta[i]==0){/* censored */
      F = cdfnorm(dta.CY[i],*m,*s);
      lp = log(1.0-F);
    } else { /* not */
      lp = d_normal_dens(dta.Y[i][0],*m,*s); 
    }
  }
  return(lp);
}

/* -----------------------------------------------------------
   check class
/* ----------------------------------------------------------- */
/* checks for bogous classes */

void check_class(){
  int i,j,k;

  for(k=0;k<mdp.n_class;k++){
    for(j=0,i=mdp.first[k]; i!=-1; j++, i = mdp.next[i])
      if (j>mdp.count[k])
	printf("\n*** Error: class %d is bogous.\n", k);
  }
}

/* -----------------------------------------------------------
   take_out
/* ----------------------------------------------------------- */
void take_out(int ind)
{
  int k_old,i,k;
  
  k = mdp.n_class-1;
  

  /* take y[ind] out of it's current class */
  k_old = mdp.member[ind];
  mdp.count[k_old] -= 1;
  mdp.updated[k_old] = 0;
  mdp.member[ind] = -1;
  if (mdp.prev[ind] == -1)
    mdp.first[k_old] = mdp.next[ind];
  else mdp.next[mdp.prev[ind]] = mdp.next[ind];
  mdp.next[ind] = -1;
  mdp.prev[ind] = -1;

  /* if class k=k_old is empty */
  if (mdp.count[k_old] == 0){

    /* relabel class k=n_class-1 as k_old */
    mdp.updated[k_old] = mdp.updated[k];
    mdp.updated[k] = 0;
    mdp.count[k_old] = mdp.count[k];
    mdp.count[k] = 0;
    mdp.first[k_old] = mdp.first[k];
    mdp.first[k] = -1;
    mdp.new[k_old] = mdp.new[k];

    if (p0>1){
      A_swap_B(mdp.V[k_old],mdp.V[k],p0,p0);
      A_swap_B(mdp.Vt[k_old],mdp.Vt[k],p0,p0);
      A_swap_B(mdp.Vj[k_old],mdp.Vj[k],p0,p0);
      A_swap_B(mdp.V_cd_inv[k_old],mdp.V_cd_inv[k],p0,p0);
      A_swap_B(mdp.Vt_cd_inv[k_old],mdp.Vt_cd_inv[k],p0,p0);   
      x_swap_y(mdp.mu[k_old],mdp.mu[k],p0);
      x_swap_y(mdp.mj[k_old],mdp.mj[k],p0);
    } else { // probably no need to swap, could just kold <- k
      r_swap_s(&mdp.mu1[k_old],&mdp.mu1[k]);
      r_swap_s(&mdp.V1[k_old],&mdp.V1[k]);
      r_swap_s(&mdp.Vt1[k_old],&mdp.Vt1[k]);
      /*r_swap_s(&mdp.mj1[k_old],&mdp.mj1[k]);*/
      r_swap_s(&mdp.Vj1[k_old],&mdp.Vj1[k]);
    }

    x_swap_y(sim.mj1[k_old],sim.mj1[k],p1);
    x_swap_y(sim.sj1[k_old],sim.sj1[k],p1);
    /*A_swap_B(sim.pij2[k_old],sim.pij2[k],p2,dta.mx_q2);*/
    x_swap_y(sim.aj4[k_old],sim.aj4[k],p4);
    x_swap_y(sim.bj4[k_old],sim.bj4[k],p4);

    /* update member-references to the relabeled class */
    for(i=mdp.first[k_old];i!=-1;i=mdp.next[i])
      mdp.member[i] = k_old;

    /* decrement n-class by one */
    mdp.n_class -= 1;
    
    check_class();

  }
}
/* -----------------------------------------------------------
   add_class
/* -----------------------------------------------------------
   adds obs[ind] to class[k]
*/
void add_class(int ind, int k)
{
  mdp.updated[k] = 0;
  mdp.count[k] += 1;
  if (mdp.count[k] == 1){
    mdp.new[k] = 1;
    mdp.n_class += 1;
  }
  mdp.member[ind] = k;
  mdp.next[ind] = mdp.first[k];
  mdp.first[k] = ind;
  if (mdp.count[k] > 1) mdp.prev[mdp.next[ind]] = ind;
  mdp.prev[ind] = -1;

  check_class();
}
/* -----------------------------------------------------------
   make_ybar
/* ----------------------------------------------------------- */
void make_ybar(int k, double *ybar)
{
  int i;
  double nk;

  nk = 1.0*mdp.count[k];
  x_zero(ybar,p0); /* initialize ybar=0 */
  for(i=mdp.first[k];i!=-1;i=mdp.next[i]){
    x_plus_y(ybar,dta.Y[i],ybar,p0);
  }
  if (mdp.count[k] != 0) x_div_r(ybar,nk,ybar,p0);
}
/* -----------------------------------------------------------
   make_Sk
/* ----------------------------------------------------------- */
void make_Sk(int k, double **S)
{
  int i,j1,j2;
  double nk, *z;

  /* allocate memory */
  z = dvector(0,p0-1);

  nk = 1.0*mdp.count[k];
  A_zero(S,p0);
  for(i=mdp.first[k];i!=-1;i=mdp.next[i]){
    x_min_y(dta.Y[i],mdp.mu[k],z,p0);
    A_plus_rxxt(S,1.0,z,S,p0);
  }
  free_dvector(z,0,p0-1);
}

double make_Sk1(int k)
{
  int i,j1,j2;
  double nk, S, ybar1;
 make_ybar(k, &ybar1);
  nk = 1.0*mdp.count[k];
  S = 0;
  for(i=mdp.first[k];i!=-1;i=mdp.next[i])
    S += sqr(dta.Y[i][0]-mdp.mu1[k]);

 /*S += sqr(dta.Y[i][0]-ybar1);*/
  return(S);
}

/* *************************************************
   setup_mu
 ************************************************* */
	
	
	
	
	
void setup_mu(int k, int draw)
{
	int j; int ii=0; double pp=0;

  double **V_inv, *ybar, nk;
  double **Vj, **Vj_inv, Vj1, ybar1;

  nk = 1.0*mdp.count[k];
  if (p0>1){

    /* allocate memory for the auxilary arrays */
    ybar = dvector(0,p0-1);
    V_inv = dmatrix(0,p0-1,0,p0-1);
    Vj_inv = dmatrix(0,p0-1,0,p0-1);
    Vj = dmatrix(0,p0-1,0,p0-1);

    if (mdp.count[k] == 0){
      B_assgn_A(Vj,mdp.B,p0);
      y_assgn_x(mdp.mj[k],mdp.mean,p0);
    }
    else{
      /* get pars for conditional posterior p(mu|...) */
      make_ybar(k, ybar);
      d_invert(p0,mdp.V[k], V_inv);
    
      /* compute posterior mean cov matrix for mu[k] */
      nn_bayes(1.0,mdp.B_inv,mdp.mean, 1.0/nk, V_inv, ybar,
	       Vj, Vj_inv, mdp.mj[k], p0);
    }

    /* keep for update_pred */
    B_assgn_A(mdp.Vj[k],Vj,dta.p0);

    /* if draw, then sample mu[k] */
    if (draw==1)
      mvnS_rand(p0,mdp.mj[k],Vj,mdp.mu[k]);

    /* Vt = (Vj + V[k]) */
    rA_plus_sB(1.0,Vj,1.0,mdp.V[k],mdp.Vt[k],p0);
    invcd(p0,mdp.Vt[k],mdp.Vt_cd_inv[k]);
    /* free memory */
    free_dvector(ybar, 0,p0-1);
    free_dmatrix(V_inv,0,p0-1,0,p0-1);
    free_dmatrix(Vj_inv,0,p0-1,0,p0-1);
    free_dmatrix(Vj,0,p0-1,0,p0-1);
  } else { // p0=1
    if(mdp.count[k]==0){
      Vj1 = mdp.B1;
      mdp.mj1[k] = mdp.mean1;
    }
    else {
      make_ybar(k, &ybar1);
		for(ii=mdp.first[k];ii!=-1;ii=mdp.next[ii]){
			pp=pp+dta.Y[ii][0];
		}

		
      /* compute posterior mean cov matrix for mu[k] */
nn_bayes1(1.0,1.0/mdp.B1,mdp.mean1, 
		1.0/nk, 1.0/mdp.V1[k],ybar1,
		&Vj1,&mdp.mj1[k]);
/* following two lines are for vague prior with mean 0 and variance pow(10,8) */

   mdp.mj1[k]= 1.0*pp/(nk+mdp.V1[k]/pow(10,8));

Vj1=1/(1/pow(10,8) +nk/mdp.V1[k]);
/* vague prior ended. The usual prior with nn_bayes and default ppmx priors  also works well */

	  /*mdp.mj1[k]=betadev(pp*nk + mdp.ab,nk- pp*nk +mdp.cd);*/ }

/*mdp.mj1[k]=pp;*/
    mdp.Vj1[k]=Vj1;
    mdp.Vt1[k] = Vj1+mdp.V1[k];
    if(draw==1)
      mdp.mu1[k]=mdp.mj1[k]+sqrt(Vj1)*norm_rand();
	  
	    }

  /* covariates */
	
	
	for(j=0;j<p1;j++){
		setup_mj1(j,k,draw);
		
	}
		
	for(j=0;j<p2;j++){
		
		setup_pij2(j,k,draw);
	}
	
	/*setup_mjj(k);*/
  for(j=0;j<p4;j++)
    setup_aj4(j,k,draw);

  mdp.updated[k] = 1;
}



void transposeMatrix(const double *source, int sourceNumRows, int sourceNumCols,
                     double *target)
{
	int sourceOffset = 0;
	int targetOffset;
	
	for (int sourceCol = 0; sourceCol < sourceNumCols; ++sourceCol) {
		targetOffset = sourceCol;
		for (int sourceRow = 0; sourceRow < sourceNumRows; ++sourceRow) {
			target[targetOffset] = source[sourceOffset++];
			targetOffset += sourceNumCols;
		}
	}
}

/* ***********************************************************
   sample_mu
/* ***********************************************************
   sample mu[k] from N(mu; m(y), T)
   where {m(y),T} = T*(nj*V-inv[k]*ybar[k] + B-inv/tau*m),
                    T-inv = nj*V-inv[k] + B-inv/tau.
*/
void sample_mu(int k)
{

  setup_mu(k,1);
}

/* ***********************************************************
   sample_V
/* *********************************************************** */
/* V[k] from W(1/V[k]; s+n[k], (sS+S[k])^-1),
   where S[k] = sum (y[i]-mu[k])(y[i]-mu[k])'
*/
void sample_V(int k)
{ double **Sk, **S1, **S1_inv_cd,
    **V_cd, **V_inv,
    V11,s1,Vinv1, Sk1, S11;
 int j1,j2;

 if (p0>1){
   /* allocate memory */
   V_cd = dmatrix(0,p0-1,0,p0-1);
   V_inv = dmatrix(0,p0-1,0,p0-1);
   Sk = dmatrix(0,p0-1,0,p0-1);
   S1 = dmatrix(0,p0-1,0,p0-1);
   S1_inv_cd = dmatrix(0,p0-1,0,p0-1);
   A_zero(V_inv,p0);

   /* Sk = sum (y[i]-mu[j])(y[i]-mu[j]) over member[i]=j */
   make_Sk(k,Sk);

   /* S1 = s*S+Sk */
   rA_plus_sB(mdp.s*1.0, mdp.S, 1.0, Sk, S1, p0);  // note p0
   cdinv(p0,S1,S1_inv_cd);                         // note p0
   wishart(mdp.s+mdp.count[k], p0, S1_inv_cd, V_inv);
   /* nuke out the off-diagonal elements - assume indep of
      obs vector and cov */

   /* compute V, V_cd, V_inv_cd etc. */
   d_chol_decomp(p0,V_inv, mdp.V_cd_inv[k]);
   d_inv_triang(p0,mdp.V_cd_inv[k],V_cd);
   ABt(V_cd,p0,p0,V_cd,p0,p0,mdp.V[k]);

   /* error check */
   if (isnan(mdp.V[k][0][0])){
     error("sample_V", "Vt is NaN.", 1);
   }

   /* allocate memory for the auxilary arrays */
   free_dmatrix(V_cd,0,p0-1,0,p0-1);
   free_dmatrix(V_inv,0,p0-1,0,p0-1);
   free_dmatrix(S1,0,p0-1,0,p0-1);
   free_dmatrix(Sk,0,p0-1,0,p0-1);
   free_dmatrix(S1_inv_cd,0,p0-1,0,p0-1);
 } else { //p0=1, univariate
   /* V[k] from G(1/V[k]; (s+n[k])/2, 1/2*(sS+S[k])),
      where S[k] = sum (y[i]-x[i]'mu[k])^2   */
   /* Sk = sum (y[i]-mu[j])(y[i]-mu[j]) over member[i]=j */
   Sk1 = make_Sk1(k);

   /* S1 = s*S+Sk */
   S11 = 0.5*(mdp.s*mdp.S1+Sk1);
   s1 = 0.5*(mdp.s+mdp.count[k]);
   Vinv1 = gamdev_ab(s1,S11);
   mdp.V1[k] = 1.0/Vinv1;
/*mdp.V1[k]=Sk1/(mdp.count[k]+.0001);*/
 }
}
    


/* ***********************************************************
   sample S
/* *********************************************************** 
   generates S from W(S; q+k*s, U)
   U = {(R/q)^-1 + s*sum V_inv[k]}^(-1)
*/
void sample_S()
{
  int k,j1,j2;
  double **U_inv, **U_cd, **V_inv;
  double s,r1,R1,Vinv,S1;

  if (p0>1){
    /* allocate memory for aux arrays */
    V_inv = dmatrix(0,p0-1,0,p0-1);
    U_inv = dmatrix(0,p0-1,0,p0-1);
    U_cd = dmatrix(0,p0-1,0,p0-1);
    A_zero(mdp.S,p0); /* initialize */

    /* initialize U_inv = (R/q)^-1 = q/R */
    rA(1.0*mdp.q, mdp.R_inv, U_inv, p0, p0); /* 1.0*q is double */
    /* note p0! */

    /* loop over all k classes, add up Vbar_inv and mu */
    s = mdp.s*1.0; /* s_double is double ! */
    for(k=0;k<mdp.n_class;k++){
      /* add (s-p0-1)*mdp.V_inv[k] */
      d_invert(p0, mdp.V[k], V_inv);         // note p0
      AplusBr(U_inv, V_inv, s, U_inv, p0);   // 
    }
  
    /* compute U_cd */
    cdinv(p0,U_inv,U_cd);

    /* generate S from Wish(q+s*k, U) */
    wishart(mdp.q+mdp.n_class*mdp.s, p0, U_cd, mdp.S);
    /* nuke out the off-diagonal elements - assume indep of
       obs vector and cov */
    invcd(p0,mdp.S,mdp.S_cd_inv);

    /* free auxilary arrays */
    free_dmatrix(V_inv,0,p0-1,0,p0-1);
    free_dmatrix(U_inv,0,p0-1,0,p0-1);
    free_dmatrix(U_cd,0,p0-1,0,p0-1);
  } else {
    /* prior:  S ~ Ga(q/2, 1/2*q/R), i.e. E(S)=R
       likl:   1/V[k] ~ Ga(s/2, 1/2*sS)
       post:   S|V    ~ Ga[1/2*(q+s*nk), 1/2(q/R + \sum s/V[k])] */
  
    s = 1.0*mdp.s;
    r1 = 0.5*(mdp.q+s*mdp.n_class);
    R1 = 0.5*(mdp.q*mdp.R1_inv);
    /* loop over all k classes, add up Vbar_inv and mu */
    for(k=0;k<mdp.n_class;k++){
      R1 += 0.5*s/mdp.V1[k];
    }
    /* generate S from Wish(q+s*k, U) */
    mdp.S1 = gamdev_ab(r1,R1);
  }

  return;
}

/* ***********************************************************
   sample m
/* *********************************************************** 
/* m from p(m|..) = N(m; m(mubar), T)
   m(ybar[k]) = T*(B-inv/tau*k*mubar + A-inv*a)
   T-inv = B-inv/tau*k + A-inv
*/
void sample_m()
{
  double *mubar, mubar1, K;
  int i;


  K = mdp.n_class; /* need double */
  if (p0>1){
    /* allocate memory */
    mubar = dvector(0,p0-1);
  
    /* make mubar = 1/K sum mu[k] */
    x_zero(mubar,p0); /* initialize mubar=0 */
    for(i=0;i<mdp.n_class;i++){
      x_plus_y(mubar,mdp.mu[i],mubar,p0);
    }
    x_div_r(mubar,K,mubar,p0);

    /* compute and sample posterior on m */
    nn_bayes_rand(1.0,mdp.A_inv, mdp.aa,
		  1.0/K, mdp.B_inv, mubar,
		  mdp.mean, p0);

    /* free memory */
    free_dvector(mubar,0,p0-1);
  } else {
    sumx(mdp.mu1,&mubar1,K);
    mubar1 *= 1.0/K;
    /* compute and sample posterior on m */
    nn_bayes1_rand(1.0,mdp.A1_inv, mdp.aa1,
		  1.0/K, 1.0/mdp.B1, mubar1,
		  &mdp.mean1);
  }
}

/* ***********************************************************
   draw B
/* *********************************************************** */
/* dummy */
/* sample B from p(B|..) =
   W(1/B; k+c, [cC + 1/tau*sum (mu[k]-m)(mu[k]-m)']^-1)
*/
void sample_B()
{ double **Sk, **S1, **S1_inv_cd,
    **B_cd, *z, Sk1, C11, Binv;
  int k,c1;

  if (p0>1){
    /* allocate memory */
    B_cd = dmatrix(0,p0-1,0,p0-1);
    Sk = dmatrix(0,p0-1,0,p0-1);
    S1 = dmatrix(0,p0-1,0,p0-1);
    S1_inv_cd = dmatrix(0,p0-1,0,p0-1);
    z = dvector(0,p0-1);
  
    /* Sk = sum (mu[k]-m)(mu[k]-m)' */
    A_zero(Sk,p0);
    for(k=0;k<mdp.n_class;k++){
      x_min_y(mdp.mu[k],mdp.mean,z,p0);
      A_plus_rxxt(Sk,1.0,z,Sk,p0);
    }

    /* S1 = s*S+Sk */
    rA_plus_sB(mdp.c*1.0, mdp.C, 1.0, Sk, S1, p0);
    cdinv(p0,S1,S1_inv_cd);
    wishart(mdp.c+mdp.n_class, p, S1_inv_cd, mdp.B_inv);
  
    /* compute B, B_cd, B_inv_cd etc. */
    d_chol_decomp(p0,mdp.B_inv, mdp.B_cd_inv);
    d_inv_triang(p0,mdp.B_cd_inv,B_cd);
    ABt(B_cd,p0,p0,B_cd,p0,p0,mdp.B);

    /* error check */
    if (isnan(mdp.B[0][0])){
      error("sample_B", "B is NaN.", 1);
    }

    /* allocate memory for the auxilary arrays */
    free_dmatrix(B_cd,0,p0-1,0,p0-1);
    free_dmatrix(S1,0,p0-1,0,p0-1);
    free_dmatrix(Sk,0,p0-1,0,p0-1);
    free_dmatrix(S1_inv_cd,0,p0-1,0,p0-1);
    free_dvector(z,0,p0-1);
  } else { //p0=1, univariate
    /* ******************************************************************
       CHECK THIS!!
    */
    /* B] from G(1/B; (c+K)/2, 1/2*(cC + Sk)),
       where Sk = sum (mu[i]-mean)^2   */
    /* Sk = sum (mu[k]-m)(mu[k]-m)' */
    Sk1=0;
    for(k=0;k<mdp.n_class;k++)
      Sk1 += (mdp.mu1[k]-mdp.mean1)*(mdp.mu1[k]-mdp.mean1);

    /* S1 = s*S+Sk */
    C11 = 0.5*(mdp.c*mdp.C1+Sk1);
    c1 = 0.5*(mdp.c+mdp.n_class);
    Binv = gamdev_ab(c1,C11);
    mdp.B1 = 1.0/Binv;
  }
}



/* ***********************************************************
   sample alpha
/* *********************************************************** 
   draw alpha and eta from:
   alpha from     pi*Gamma(a0+n_class, b0-log(eta)) +
              (1-pi)*Gamma(a0+n_class-1, b0-log(eta)),
         where pi = (a+n-class-1)/(a0+n_class-1 + n*b0 - n*log(eta)).
   eta   from Beta(alpha+1,n).
*/
void sample_alpha()
{
     double a,  b, pi;       /* parameters in Gamma call */
     double u;

      dbg = 0;
      if (dbg) message("sample alpha");

     /* draw eta */
     mdp.eta = betadev(mdp.alpha+1.0, n*1.0);

     /* compute pars for gammas for alpha */
     b = mdp.b0 - log(mdp.eta);
     pi = (mdp.a0 + mdp.n_class - 1)/
       (mdp.a0 + mdp.n_class -1 + n*(mdp.b0 - log(mdp.eta)));
     duniform(1,&u);
     a = (u<pi) ? mdp.a0+mdp.n_class : mdp.a0+mdp.n_class-1;

     /* draw alpha with prob pi from Gamma(a1,b) else Gamma(a2,b) */
     mdp.alpha = gamdev_ab(a,b);

     if (dbg){
       messdouble("alpha: ",mdp.alpha);
     }
     dbg=0;

}

/* ============================================================= 
   updating the similarity parameters
/* ============================================================= */ 


void setup_mj1(int j, int k, int sample){
   /* update (mu,V) for cluster l, j-th continuous covariate */
  int i;
  double nk, ybar, mu, V, m1,s1;

  if ( (j>=p1) | (k>mdp.n_class))
    err_msg("setup_mj1","j=%d or k=%d out of range.",j,k,0);

  V = sim.V1[j];
  /* get ybar */
  nk = 1.0*mdp.count[k];
  if (nk>0){
    ybar = 0;
    for(i=mdp.first[k];i!=-1;i=mdp.next[i]){
      ybar += dta.Y[i][p0+j];
    }
    if (mdp.count[k] != 0) ybar = ybar/(1.0*nk);

    nn_bayes1(1.0, 1.0/sim.B1[j], sim.mean1[j],
	      1.0/nk, 1.0/V, ybar,
	      &s1, &m1);
  } else{
    m1=sim.mean1[j];
    s1=sim.B1[j];
  }
  sim.mj1[k][j] = m1; 
  sim.sj1[k][j] = s1;
  // ignore sample request -- don't save mu1[j]
}



void setup_pij2(int j, int k, int sample){
  /* update pi3 for cluster k, j-th categorical covariate */
  int i,h,q,nk,nj;
  int yi,*y;
  double yid, s;

  if ( (j>=p2) | (k>mdp.n_class))
    err_msg("setup_pij2","j=%d or k=%d out of range.",j,k,0);
  

  q = dta.q2[j];
  y = ivector(0,q);

  /* get ybar */
  nk = 1.0*mdp.count[k];
  nj=0; /* same as nk, for plausiblity check */
  i_zero(y,q);
  if (nk>0){
    for(i=mdp.first[k];i!=-1;i=mdp.next[i]){
      yid = dta.Y[i][p0+p1+j];
     if (yid-floor(yid) > 0)
	err_msg("setup_pi2","Y[%d,%d] is not integer!",i,p0+p1+j,0);
      yi=yid;
      if ((yi<0) | (yi>q))
	err_msg("setup_pi2","Y[%d,%d]=%d out of range for variable",
		i,p0+p1+j,yi);
    if(dta.mis[i][j]==1){
      y[yi] += 1;
      nj+=1;
}
    }
    /*if (nj != nk)
     err_msg("setup_pi2","cluster %d: size=%d, does not match count[j]=%d",
	      k,nj,nk);*/
  }
  
  for(s=0,h=0;h<q;h++){
    sim.pij2[k][j][h] = sim.pi2[j][h]+y[h];
    s+= sim.pij2[k][j][h];
  }
  sim.pij2[k][j][q]=s; /* save sum in last element */

  free_ivector(y,0,q);

}

void setup_aj4(int j, int k, int sample){
  /* update pi3 for cluster k, j-th count covariate */
  int i,yj;
  int nj,Rj;
  double a1,b1,yjd;

  if ( (j>=p4) | (k>mdp.n_class))
    err_msg("setup_aj4","j=%d or k=%d out of range.",j,k,0);

  Rj=0;
  nj=0;
  if(mdp.count[k]>0){
    for(i=mdp.first[k];i!=-1;i=mdp.next[i]){
      yjd = dta.Y[i][p0+p1+p2+p3+j];
      if (yjd-floor(yjd) > 0)
	err_msg("setup_aj4","Y[%d,%d] is not integer!",
		i,p0+p1+p2+p3+j,0);
      yj=yjd;
      if (yjd<0)
	err_msg("setup_aj4","y=%d <0",yj,0,0);
      Rj += yjd;
      nj+=1;
    }
    if (nj != mdp.count[k])
      err_msg("setupu_aj4","cluster %d: size=%d does not match count[j]=%d",
	      k,nj,mdp.count[k]);
  }
  sim.aj4[k][j]=sim.a4[j]+Rj;
  sim.bj4[k][j]=sim.b4[j]+nj;
}


/* ============================================================= 
   printing routines
/* ============================================================= */ 

/* ***********************************************************
   print pi
/* ***********************************************************
   prints current pi's */
void print_pi()
{
     int k, j, i;

     
     muFile = openAppend("mu.mdp");
     for (k = 0; k < mdp.n_class; k++){
       /*fprintf(muFile," %4d",iter);
       fprintf(muFile,"\t %4d \t", mdp.count[k]);*/
       if (p0>1)
	 fwriteDoubleArray(muFile, mdp.mu[k], 1, p0);
       else 
{fprintf(muFile,"%6.3f ", mdp.mj1[k]); fprintf(muFile,"\t %4d \t", iter);
 fprintf(muFile,"\n");}
     }
     fprintf(muFile,"\n");
     fclose(muFile);

     VFile = openAppend("V.mdp");
     for (k = 0; k < mdp.n_class; k++){
       /*fprintf(VFile,"\n %4d",iter);
       fprintf(VFile,"\t %4d\n", mdp.count[k]);*/
       if (p0>1)
	 fwriteDoubleMatrix2(VFile,mdp.V[k], p0, p0);
       else 
{fprintf(VFile,"%6.3f ", mdp.Vt1[k]); fprintf(VFile,"\t %4d \t", iter);
 fprintf(VFile,"\n");}

    }
     fclose(VFile);

     memberFile = openAppend("member.mdp");
       fprintf(memberFile," %4d",iter);
       fprintf(memberFile," %4d \t", mdp.n_class);
       for(i=0;i<n;i++)
	 fprintf(memberFile," %3d", mdp.member[i]);
     fprintf(memberFile,"\n");
     fclose(memberFile);

     if (p1>0){ //continous covariates
       mj1file=openAppend("mj1.mdp");
       fprintf(mj1file,"\n %4d %4d ",iter,mdp.n_class);
       for(k=0;k<mdp.n_class;k++){
	 fwriteDoubleArray(mj1file, sim.mj1[k], 1, p1);
	 fwriteDoubleArray(mj1file, sim.sj1[k], 1, p1);
       }
       fclose(mj1file);
     }
     if (p2>0){ // categorical covariates
       pij2file=openAppend("pij2.mdp");
       /*fprintf(pij2file,"\n %4d %4d ",iter,mdp.n_class);*/
       for(k=0;k<mdp.n_class;k++){
	 for(j=0;j<p2;j++){
fprintf(pij2file, "%4.2f ",sim.pij2[k][j][0]); fprintf(pij2file, "\t %4.2f \t",sim.pij2[k][j][1]); fprintf(pij2file , " \t %4d \t", iter);
fprintf(pij2file , " \t %4d \t", (j+1) ); fprintf(pij2file,  "\n");

}
           

	   /*fwriteDoubleArray(pij2file, sim.pij2[k][j],1,dta.q2[j]);*/
       } // for k
       fclose(pij2file);
     }
     if (p4>0){ // ordinal covariates
       aj4file=openAppend("aj4.mdp");
       fprintf(aj4file,"\n %4d %4d ",iter,mdp.n_class);
       for(k=0;k<mdp.n_class;k++){
	 fwriteDoubleArray(aj4file, sim.aj4[k],1,p4);
	 fwriteDoubleArray(aj4file, sim.bj4[k],1,p4);
       } // for k
       fclose(aj4file);
     }
       
     
     
}

/* ***********************************************************
   print pars
/* ***********************************************************
   print current tau,alpha,eta */
void print_pars(int time)
{
  static int linect=20;
  int i,k, ct[3], k1[3];
  
  if (mdp.verbose < 2){
    printf("%d:%d ",iter,mdp.n_class);
    if (iter % 10 == 0) printf("\n");
    return;
  }

  /* print header line if linect == 20 */
  if (linect == 20){
    if (B_prior) 
      printf("\n\n\n %5s %16s %5s ", "iter","k (count1,2,3)  ","alpha");
    else
      printf("\n\n\n %5s %16s %5s %5s ", "iter","k (count1,2,3)  ",
	     "alpha", "tau");
    for(i=0;i<p0;i++) printf("mean[%1d](S,B)    ",i);
    if (mdp.verbose){
      for(k=0;(k<3)&(k<mdp.n_class);k++) 
	for(i=0;i<p0;i++) printf("mu[%1d][%1d]    ",k,i);
      printf
	("\n----------------------------------------------------",
	 "-------------------------- \n");
      linect = 0;
    }
    printf("\n");
  }
  /* find largest 3 classes */
  k1[0] = k1[1] = k1[2] = 0;
  ct[0] = ct[1] = ct[2] = 0;
  for(k=0; k<mdp.n_class; k++){
    if (mdp.count[k] > ct[0]){
      k1[2] = k1[1];
      ct[2] = ct[1];
      k1[1] = k1[1];
      ct[1] = ct[0];
      k1[0] = k;
      ct[0] = mdp.count[k];
    }
    else if (mdp.count[k] > ct[1]){
      k1[2] = k1[1];
      ct[2] = ct[1];
      k1[1] = k;
      ct[1] = mdp.count[k];
    }
    else if (mdp.count[k] > ct[2]){
      k1[2] = k;
      ct[2] = mdp.count[k];
    }
  }

  /* print line */
  if (B_prior)
    printf(  "%5d %3d(%3d %3d %3d) %5.2f ",
	   iter,mdp.n_class,ct[0], ct[1], ct[2],
	   mdp.alpha);
  else
    printf(  "%5d %3d(%3d %3d %3d) %5.2f ",
	   iter,mdp.n_class,ct[0], ct[1], ct[2],
	     mdp.alpha);
  if (p0>1)
    for(i=0;i<p0;i++) 
      printf("%4.1f(%4.1f,%4.1f) ", mdp.mean[i], 
			     sqrt(mdp.S[i][i]), sqrt(mdp.B[i][i]));
  else 
    printf("%4.1f(%4.1f,%4.1f) ", mdp.mean1,
			     sqrt(mdp.S1), sqrt(mdp.B1));
  if (mdp.verbose){
    for(k=0;(k<3)&(k<mdp.n_class);k++){
      if (mdp.new[k1[k]] == (iter-1)) printf("* ");
      else printf("| ");
      if (p0>1)
	for(i=0;i<p0;i++) 
	  printf("%4.1f(%4.1f) ",
			      mdp.mu[k1[k]][i],sqrt(mdp.V[k1[k]][i][i]));
      else
	  printf("%4.1f(%4.1f) ",mdp.mu1[k1[k]],sqrt(mdp.V1[k1[k]]));
    }
  }
  printf("\n");
  linect += 1;
}


int print_allpars()
{
  int k;

  if (p0>1){
    messdoublematrix2("B ",mdp.B,p0,p0);
    messdoublematrix2("S ",mdp.S,p0,p0);
    messdoublevec("m ",mdp.mean,p0);
    for(k=0;k<mdp.n_class;k++){
      messdoublevec("mu[i] ",mdp.mu[k],p0);
      messdoublematrix2("V[j] ",mdp.V[k],p0,p0);
    }
    printf("Note: sampling seq is \t  S,config,mu,V,B,m.\n");
    printf("------------------------------------------------------- \n");
  }
}

/* ***********************************************************
   write pars
/* ***********************************************************
   print current tau,alpha,eta */
void write_pars(int time)
{
  parFile = openAppend("par.mdp");
  fprintf(parFile,"%4d  %6.3f %6.3f %4d\n",
	  iter, mdp.alpha,mdp.eta, mdp.n_class);
  fclose(parFile);
  meanFile = openAppend("mean.mdp");
  fprintf(meanFile,"%4d \t", iter);
  if (p0>1) fwriteDoubleArray(meanFile, mdp.mean, 1, p0);
  else 	    fprintf(meanFile, "%6.3f", mdp.mean1);
  fclose(meanFile);
  SFile = openAppend("S.mdp");
  fprintf(SFile,"%4d \t", iter);
  if (p0>1)
    fwriteDoubleMatrix2(SFile, mdp.S, p0,p0);
  else
    fprintf(SFile, "%6.3f ", mdp.S1);
  fclose(SFile);
  if (B_prior){
    BFile = openAppend("B.mdp");
    fprintf(BFile,"%4d \t", iter);
    if (p0>1)
      fwriteDoubleMatrix2(BFile, mdp.B, p0,p0);
    else
      fprintf(BFile, "%6.3f ", mdp.B1);
    fclose(BFile);
  }
}


/* ============================================================= 
  ergodic avges for group numbers
/* ============================================================= */ 

int nclass_initialized=0; /* indicator whether nclass is initialized */
double *nclass; /* accumulate number of clusters */
int max_nclass = 0;

/* ***********************************************************
   update_nclass
/* *********************************************************** */
int update_nclass()
{
  int j;

  if (nclass_initialized == 0){
    nclass = dvector(0,n);
    for (j=0;j<=n;j++)
      nclass[j] = 0.0;
    nclass_initialized = 1;
  }

  max_nclass = (mdp.n_class > max_nclass) ? mdp.n_class : max_nclass;
  nclass[mdp.n_class] += 1.0;
}


int write_nclass()
{
  int j;
  FILE *pjfile;

  pjfile = openOut("pj.mdp");
  fwriteDoubleArray(pjfile, nclass, 10, max_nclass);
  fclose(pjfile);
}
