static const char regr_c_sccs_id[] = "%W%";

/*+
 * regr.c - Regression
 *
 * Description:
 *
 * Modification History:
 * <unknown>	1.1		Peter Mueller
 *		Created.  Corrected on 20/6/96 with:
 *			logit_setup:        added normal error, 
 *			logit_bayes_setup   same,
 *			lp_logistic2()		corrected normal pdf!!! 
 *
 * 29-JUN-1999	1.3		Stephen Morris (Tessella Support Services plc)
 *		Added this header.
 *		Removed embedded comments (slash-star in a comment block)
 *		Include bayes.h to define nn_bayes, and stdlib.h for exit().
+*/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bayes.h"
#include "interface.h"
#include "vector.h"
#include "matrix.h"
#include "nrutil.h"
#include "rand.h"
#include "regr.h"
#include <R.h>
#include <Rdefines.h>
#include <Rmath.h>
#include <R_ext/Utils.h>


/* *************************************************
 *
 *   multiple regression
 *
 * *************************************************
   computes mean and cov matrix for:
MODEL
  y    = x'beta + eps, with eps ~ N(0,h[t])

ARGUMENTS
 mu            mean vector of normal posterior
 V=H_inv_cd,
 L=H_cdt       cov matrix: S=VV', L=1/V
 y             dep vars
 h             varianes (or inverse weights)
 x             matrix of covariates
 n             n obs
 p             n covariates
*/

/* *************************************************
   multiple regressiuon
 ************************************************* */

void regr2(double *yh, 
		  double *y, double **x, int n, int p,
		  int var_known, double sig)
{
	double *xy, W, SS, yhat,
    **H, *h,  **H_inv,  **H_cdt, **H_inv_cd;
	int 
    i,j,k;
	double *mu;
	
	mu=dvector(0.0,p-1);
	/*yh=dvector(0.0, n-1);
	/* allocate memory */
	xy = dvector(0,p-1);
	H = dmatrix(0,p-1,0,p-1);
	H_inv = dmatrix(0,p-1,0,p-1);
	H_inv_cd = dmatrix(0,p-1,0,p-1);
	H_cdt = dmatrix(0,p-1,0,p-1);
	
	for(j=0;j<p;j++){
		xy[j] = 0.0;
		for(k=0;k<p;k++)
			H[j][k] = 0.0;
	}
	
	for(i=0;i<n;i++){
		/*W = (var_known==1) ? h[i] : 1.0;*/
		W=1;
		for(j=0;j<p;j++){
			/*Rprintf("%f \t",x[i][j]);*/
			for(k=0;k<p;k++)
			{
				H[j][k] += x[i][j]*x[i][k]/W;
				if (H[j][k] == 0)
				{
					j = j;
				}
			}
		}
		/*Rprintf("\n");*/
		for(j=0;j<p;j++)      
			xy[j] += x[i][j]*y[i]/W;
	}
	
	/* compute V_cd */
	d_chol_decomp(p,H,H_cdt); 
	d_inv_triang(p,H_cdt,H_inv_cd);
	ABt(H_inv_cd,p,p,H_inv_cd,p,p,H_inv);
	Ax(H_inv,xy,mu,p);
	/*Rprintf("ok5");
	/* compute s2 if var is unkwnown */
	/*Rprintf("\t %f \t %f \t %f", mu[0],mu[1],mu[2]);*/
	if (var_known==0){
		for(i=0,SS=0;i<n;i++){
			for(j=0,yhat=0.0;j<p;j++)
				yhat += x[i][j]*mu[j];
			yh[i]=yhat;
			SS += (yhat-y[i])*(yhat-y[i]);
		}
		sig = SS/(n-p*1.0);
	}
	else sig=1.0;
	
	rA(sqrt(sig),H_inv_cd,H_inv_cd,p,p);
	rA(1.0/sqrt(sig),H_cdt,H_cdt,p,p);
	
	/* free memory */
	free_dvector(xy,0,p-1);
	
	free_dmatrix(H, 0,p-1,0,p-1);
	free_dmatrix(H_inv, 0,p-1,0,p-1);
	
}












void regr(double *mu, double **H_inv_cd, double **H_cdt,
	  double *y, double *h, double **x, int n, int p,
	  int var_known)
{
  double 
    *xy, W, sig, SS, yhat,
    **H, **H_inv;
  int 
    i,j,k;

  /* allocate memory */
  xy = dvector(0,p);
  H = dmatrix(0,p,0,p);
  H_inv = dmatrix(0,p,0,p);

  for(j=0;j<p;j++){
    xy[j] = 0.0;
    for(k=0;k<p;k++)
      H[j][k] = 0.0;
  }

  for(i=0;i<n;i++){
    W = (var_known==1) ? h[i] : 1.0;
    for(j=0;j<p;j++)
      for(k=0;k<p;k++)
	{
	  H[j][k] += x[i][j]*x[i][k]/W;
	  if (H[j][k] == 0)
	    {
	      j = j;
	    }
	}
    for(j=0;j<p;j++)      
      xy[j] += x[i][j]*y[i]/W;
  }

  /* compute V_cd */
  d_chol_decomp(p,H,H_cdt); 
  d_inv_triang(p,H_cdt,H_inv_cd);
  ABt(H_inv_cd,p,p,H_inv_cd,p,p,H_inv);
  Ax(H_inv,xy,mu,p);

  /* compute s2 if var is unkwnown */
  if (var_known==0){
    for(i=0,SS=0;i<n;i++){
      for(j=0,yhat=0.0;j<p;j++)
	yhat += x[i][j]*mu[j];
      SS += (yhat-y[i])*(yhat-y[i]);
    }
    sig = SS/(n-p*1.0);
  }
  else sig=1.0;

  rA(sqrt(sig),H_inv_cd,H_inv_cd,p,p);
  rA(1.0/sqrt(sig),H_cdt,H_cdt,p,p);

  /* free memory */
  free_dvector(xy,0,p);

  free_dmatrix(H, 0,p,0,p);
  free_dmatrix(H_inv, 0,p,0,p);
}

/* *************************************************
   simple regr
 ************************************************* */

void regr1(double *mu, double *sd, double *prec,
	  double *y, double *h, double *x, int n, 
	  int var_known)
{
  double 
    xy=0, W, sig, SS, yhat, xtx=0;
  int 
    i;

  /* compute xtx = x'x = sum(x*x), xy = x'y = sum(x*y) */
  for(i=0,xtx=0,xy=0;i<n;i++){
    W = (var_known==1) ? h[i] : 1.0;
    xtx += x[i]*x[i]/W;
    xy  += x[i]*y[i]/W;
  }

  /* mu = (x'x)^-1 x'y */
  *mu = xy/xtx;

  /* compute var if h[i]'s are unknown */
  if (var_known==0){
    for(i=0,SS=0;i<n;i++){
      yhat = x[i]*(*mu);
      SS += (yhat-y[i])*(yhat-y[i]);
    }
    sig = SS/(n-1.0);
  }
  else sig=1.0;

  /* poserior sd and prec */
  *sd = sqrt(sig/xtx);
  *prec = 1.0/(*sd);

}


/* *************************************************
   bayes multiple regressiuon
 ************************************************* */
void bayes_regr(double *mu2, double **V, double **L,
	  double *y, double *h, double **x, 
	  double *mu1, double **S1_inv, int n, int p,
	  int var_known)
{
  double 
    *b,**S2_inv,**S2;
  int 
    i;
  double 
    *xy, W, **H;

  /* var unknown not yet implemented ! */
  if (var_known==0){
    printf(
   "\n*** Error in bayes_regr: unknown var not yet impl\n");
    exit(1);
  }

  /* alloc mem */
  S2_inv = dmatrix(0,p,0,p);
  S2 = dmatrix(0,p,0,p);
  b = dvector(0,p);
  xy = dvector(0,p);
  H = dmatrix(0,p,0,p);

  /* get likelihood from regression */
  x_zero(xy,p);
  A_zero(H,p);

  for(i=0;i<n;i++){
    W = (var_known==1) ? h[i] : 1.0;
    A_plus_rxxt(H,1.0/W,x[i],H,p);
    x_plus_ry(xy,y[i]/W,x[i],xy,p);
  }

  /* S2_inv = S1_inv + H*sig^(-2) */
  A_plus_B(S1_inv,H,S2_inv,p,p);
  Ax_plus_y(S1_inv,mu1,xy,b,p);

  /* compute V_cd */
  d_chol_decomp(p,S2_inv,L);
  d_inv_triang(p,L,V);
  ABt(V,p,p,V,p,p,S2);
  Ax(S2,b,mu2,p);

  /* free memory */
  free_dvector(xy,0,p);
  free_dmatrix(H, 0,p,0,p);
  free_dmatrix(S2_inv ,0,p,0,p);
  free_dmatrix(S2 ,0,p,0,p);
  free_dvector(b,0,p);
}

void bayes_regr_rand(double *th,
	  double *y, double *h, double **x, 
	  double *mu1, double **S1_inv, int n, int p,
	  int var_known)
{
  double *m2,**V,**L;

  /*alloc mem */
  m2 = dvector(0,p);
  V = dmatrix(0,p,0,p);
  L = dmatrix(0,p,0,p);

  bayes_regr(m2,V,L,y,h,x,mu1,S1_inv,n,p,var_known);
  mvn_rand(p,m2,V,th);

  free_dvector(m2,0,p);
  free_dmatrix(V,0,p,0,p);
  free_dmatrix(L,0,p,0,p);
}


/* -------------------------------------------------
   old version
 ------------------------------------------------- */
void bayes_regr_old(double *mu2, double **V, double **L,
	  double *y, double *h, double **x, 
	  double *mu1, double **S1_inv, int n, int p,
	  int var_known)
{
  double 
    *mu,**Slikl_inv,**S2_inv,**S2;

  /* alloc mem */
  Slikl_inv = dmatrix(0,p,0,p);
  S2_inv = dmatrix(0,p,0,p);
  S2 = dmatrix(0,p,0,p);
  mu = dvector(0,p);

  /* get likelihood from regression */
  regr(mu,V,L,y,h,x,n,p,var_known);
  AtB(L,p,p,L,p,p,Slikl_inv);

  nn_bayes(1.0,S1_inv,mu1,1.0,Slikl_inv,mu,S2,S2_inv,mu2,p);
  d_chol_decomp(p,S2_inv,L);
  d_inv_triang(p,L,V);

  /* free mem */
  free_dmatrix(Slikl_inv ,0,p,0,p);
  free_dmatrix(S2_inv ,0,p,0,p);
  free_dmatrix(S2 ,0,p,0,p);
  free_dvector(mu,0,p);
}


/* *************************************************
   bayes simple regr
 ************************************************* */

void bayes_regr1(double *mu, double *sd, double *prec,
	  double *y, double *h, double *x,
	  double mu1, double s1, int n)
{
  double 
    xy=0, W, sig, xtx=0, mu2, s2, spost;
  int 
    i;

  /* compute xtx = x'x = sum(x*x), xy = x'y = sum(x*y) */
  for(i=0,xtx=0,xy=0;i<n;i++){
    W = h[i];
    xtx += x[i]*x[i]/W;
    xy  += x[i]*y[i]/W;
  }

  /* mu = (x'x)^-1 x'y */
  mu2 = xy/xtx;

   sig=1.0;

  /* poserior sd and prec */
  s2 = sig/xtx;
  
  spost = 1.0/(1.0/s1 + 1.0/s2);
  *sd = sqrt(spost);
  *mu = (mu1/s1 + mu2/s2)*spost;
  *prec = 1.0/(*sd);
}





/* *************************************************
 *
 *   indep chain for logistic regr
 * *************************************************

 V,L           cov matrix: S=VV', L=1/V
 p:            dim par vector
 N             binomial N (=0 for normal)
 n             n obs
 sig           var for normal error (ignored for binom)
*/

void logit_setup(double *mu, double **V, double **L,
		 double *y, double **x, double *offset, int N,
		 int n, int p, double *sig)
{
  double 
    e,D,W, eta0, mu0; /* auxilary */
  double 
    *dmu, **H, **H_cdt, **H_inv_cd, **H_inv, 
    *z, *h;
  int 
    i,j,k, normal /* indicator for normal */;

  /* allocate memory */
  h = dvector(0,p);
  dmu = dvector(0,p);
  z = dvector(0,n);

  H = dmatrix(0,p,0,p);
  H_cdt = L; 
  H_inv_cd = V;
  H_inv = dmatrix(0,p,0,p);

  normal = (N==0); /* normal error */
  if (normal)
    N=1; /* to avoid splitting cases.. */

  for(j=0;j<p;j++){
    h[j] = 0.0;
    for(k=0;k<p;k++)
      H[j][k] = 0.0;
  }

  for(i=0;i<n;i++){
    /* -------------------------------------------------
       (1) compute z
       ------------------------------------------------- */
    xy(mu,x[i],&eta0,p);
    eta0 += offset[i];
    e = exp(-eta0);
    mu0 = N*1.0/(1.0+e);
    if (mu0 > 0.999*N) mu0 = 0.999*N;
    if (mu0 < 0.001*N) mu0 = 0.001*N; 
    if (normal){
      D = 1.0/(mu0*(1-mu0));
      W = D*D*sig[i];
    }
    else {
      D = 1.0/(mu0*(1.0-mu0/N));
      W = D*D*mu0*(1.0-mu0/N); /* could simplify .. */
    }
    z[i] = D*(y[i]-mu0);

    /* -------------------------------------------------
       (2) regression
       ------------------------------------------------- */
    for(j=0;j<p;j++)
      for(k=0;k<p;k++)
	H[j][k] += x[i][j]*x[i][k]/W;
    for(j=0;j<p;j++)
      h[j] += x[i][j]*z[i]/W;
  }
  /* compute V_cd */
  d_chol_decomp(p,H,H_cdt); 
  d_inv_triang(p,H_cdt,H_inv_cd);
  ABt(H_inv_cd,p,p,H_inv_cd,p,p,H_inv);
  Ax(H_inv,h,dmu,p);

  for(j=0;j<p;j++)
    mu[j] += dmu[j];

  /* free memory */
  free_dvector(h,0,p);
  free_dvector(dmu,0,p);
  free_dvector(z,0,n);

  free_dmatrix(H, 0,p,0,p);
  free_dmatrix(H_inv, 0,p,0,p);
}

/* -------------------------------------------------
   bayes_logit_setup
 ------------------------------------------------- */


void bayes_logit_setup2(double *mu, double **V, double **L,
		       double *m1, double **S1_inv,
		       double *y, double **x, double *offset,
		       int n, int p, double *sig, int normal)
{
  float e,D,W, eta0, mu0; /* auxilary */
  double 
    *dmu, **H,
    *z, *h, *dmmy,
    **S2, **S2_inv, *dm1, *b;
  int 
    i,j,k;

  /* allocate memory */
  h = dvector(0,p-1);
  dmu = dvector(0,p-1);
  dm1 = dvector(0,p-1);
  b = dvector(0,p-1);
  dmmy = dvector(0,100);
  z = dvector(0,n-1);

  H = dmatrix(0,p-1,0,p-1);
  S2 = dmatrix(0,p-1,0,p-1);
  S2_inv = dmatrix(0,p-1,0,p-1);

  /* had here: if (normal) i_assgn_r(N2,1,n); */

  /* initialize to zero */
  for(j=0;j<p;j++){
    h[j] = 0.0;
    for(k=0;k<p;k++)
      H[j][k] = 0.0;
  }

  /* prior mean for mu - mu0 */
  x_min_y(m1,mu,dm1,p);

  for(i=0;i<n;i++){
   /* if ((!normal)&(N[i]==0)) 
      continue; /* void observation - allowed to be included */
    /* -------------------------------------------------
       (1) compute z
       ------------------------------------------------- */
    /*xy(mu,x[i],&eta0,p);*/
	  eta0=0;
	  for(j=0;j<p;j++){
		  /*if(i==0) {printf("muj is %f",mu[j]);}*/
		  eta0=eta0+x[i][j]*mu[j];
	  }
	  /*if(i==0) printf("\n");*/
	 
   /* eta0 += offset[i];*/
    e = exp(-eta0);
    if (normal){
      mu0 = 1.0/(1.0+e);
      if (mu0 > 0.999) mu0 = 0.999;
      if (mu0 < 0.001) mu0 = 0.001;
      D = 1.0/(mu0*(1-mu0));
      W = D*D*sig[i];
    }
    else {
      mu0 = 1.0/(1.0+e);
      if (mu0 > 0.999) mu0 = 0.999;
      if (mu0 < 0.001) mu0 = 0.001; 
      D = 1.0/(mu0*(1.0-mu0));
      W = D*D*mu0*(1.0-mu0); /* could simplify .. */
    }
    z[i] = D*(y[i]-mu0);

    /* -------------------------------------------------
       (2) regression
       ------------------------------------------------- */
	  W=1/(mu0*(1.0-mu0));
    for(j=0;j<p;j++)
      for(k=0;k<p;k++)
	H[j][k] += x[i][j]*x[i][k]/W;
    for(j=0;j<p;j++)
      h[j] += x[i][j]*z[i]/W;
  }
 /*printf("\t %f \t  %f \t %f ", W,eta0,mu0);*/
  /* S2_inv = S1_inv + H*sig^(-2) */
  A_plus_B(S1_inv,H,S2_inv,p,p);
  Ax_plus_y(S1_inv,dm1,h,b,p);
  /* note: dm1 instead of mu1! */

  /* compute V_cd */

	for(j=0;j<p;j++){
		for(k=0;k<p;k++){
			/*printf("\t %f", S2_inv[j][k]);*/
		}
	}

  d_chol_decomp(p,S2_inv,L);
  d_inv_triang(p,L,V);
  ABt(V,p,p,V,p,p,S2);
  Ax(S2,b,dmu,p);

  for(j=0;j<p;j++)
    mu[j] += dmu[j];

  /* free memory */
  free_dvector(dm1,0,p-1);
  free_dvector(b,0,p-1);
  free_dvector(h,0,p-1);
  free_dvector(dmu,0,p-1);
  free_dvector(z,0,n-1);

  free_dvector(dmmy,0,100);

  free_dmatrix(S2_inv, 0,p-1,0,p-1);
  free_dmatrix(H, 0,p-1,0,p-1);
  free_dmatrix(S2, 0,p-1,0,p-1);
}

/* -------------------------------------------------
   lp logistic
 ------------------------------------------------- */
/* returns logistic log pdf */


void matrix_times_vector(double **a, double *x, double *result, int nrow, int ncol)
{
    register int i;
	
    for (i = 0; i < nrow; i++) {
        register int j;
        double sum;
		
        sum = 0.0;
        for (j = 0; j < ncol; j++) {
            sum = sum + a[i][j] * x[j];
        }
        result[i] = sum;
    }
}



double lp_logistic2(double *beta, double *y, double **x, 
		    double *offset,int n, int p, double *sig,
		    int normal)
{
  int 
    i;
  double 
    lp,pi,eta;

  for(i=0,lp=0.0;i<n;i++){
   /* if ((!normal) & (N[i]==0))
      continue; /* void observation */
    xy(beta,x[i],&eta,p);
    eta += offset[i];
    pi = 1.0/(1.0+exp(-eta));
    if (normal)
      lp += -0.5*(y[i]-pi)*(y[i]-pi)/sig[i];
    /* CORRECTED from missing "-0.5*" !!!! */
    else{
      if ( (pi==0.0) | (pi == 1.0) ) lp = -10e10;
      else
      lp += y[i]*log(pi) + (1-y[i])*(log(1.0-pi));
    }
  }
  return (lp);
}

/* *************************************************;
   regr_logistic
 ************************************************* 
PROGRAM FLOW
   Loop over n_iter iterations
   (1) if necesary, update approximation
   (2) draw bnew from N(mub_hat, sigb_hat^2)
   (3) compare p(bnew,..) with p(b,...) and accept with
       appropriate prob (independence chain)
ARGUMENTS
 n_iter        number its in indep chain
 beta:         initial guess
 p:            dim par vector
 n:            n obs
 y:            0/1 obs
 x             covariates
 n_jump        incremented by 1 if jump accepted, unchanged else
*/
/* -------------------------------------------------
   bayes_regr_logistic
 ------------------------------------------------- */
/*void bayes_regr_logistic(double *beta, 
   double *m1, double **S1_inv,		       
   double *y, double **x, double *offset, int N,
   int n, int p, double *sig, int n_iter, int *n_jump)
{ /* old version of bayes_regr_logistic2: kept for compatibility 
  int *Nvec, normal;
  Nvec = ivector(0,n);

  i_assgn_r(Nvec,N,n);
  normal = (N==0) ? 1 : 0;
  bayes_regr_logistic2(beta, m1, S1_inv, y, x, offset, Nvec,
		       n,  p, sig, n_iter, n_jump, normal);
  free_ivector(Nvec,0,n);
}*/

void bayes_regr_logistic2( double *res, double *beta, 
   double *y, double **x, double *offset,
   int n, int p, double *sig, int n_iter,  int normal)
{
  int 
    it; int iu;

  double
    *e, *bnew,lpr,lprz,
    qz,q,lp,lpz,dpdq,u,probjump,
    **L, **Ltilde, **V, *mu_hat, *mu_hattilde, **L1;

	double *m1; double **S1_inv;
	S1_inv=dmatrix(0,p-1,0,p-1);
	m1=dvector(0,p-1); 
	for(iu=0;iu<p;iu++){
		S1_inv[iu][iu]=.0001;
		
	}
  /* allocate memory */
  L = dmatrix(0,p-1,0,p-1);
  L1 = dmatrix(0,p-1,0,p-1);
  V = dmatrix(0,p-1,0,p-1);
  mu_hat = dvector(0,p-1);
  Ltilde = dmatrix(0,p-1,0,p-1);
  mu_hattilde = dvector(0,p-1);
  e = dvector(0,p-1);
  bnew = dvector(0,p-1);

 d_chol_decomp(p,S1_inv,L1);
	
  lp = lp_logistic2(beta,y,x,offset,n,p,sig,normal);
	lpr=1;
  lpr = mvn_logpdf(beta,m1,L1,p); /* log prior */

  y_assgn_x(mu_hat,beta,p);
	/*Rprintf("ok 4a");*/
	bayes_logit_setup2(mu_hat,V,L,m1,S1_inv,y,x,offset,
		     n,p,sig,normal); /* V is dummy only */
	/*Rprintf("ok 4b");*/
  for(it=0;it<n_iter;it++){
    /* -------------------------------------------------
       (2) draw new b1..6
       ------------------------------------------------- */
    dstdnormal(p,e);
    xtx(e,&qz,p);
    qz *= -0.5;
    Ax_plus_y(V,e,mu_hat,bnew,p);
    /* -------------------------------------------------
       (1) update approx
       ------------------------------------------------- */
    y_assgn_x(mu_hattilde,bnew,p);
    bayes_logit_setup2(mu_hattilde,V,Ltilde,m1,S1_inv,y,x,offset,
      n,p,sig,normal); /* V is dummy only */
    
    /* -------------------------------------------------
       (3) compare
       ------------------------------------------------- */
    lpz = lp_logistic2(bnew,y,x,offset,n,p,sig,normal);
    lprz = mvn_logpdf(bnew, m1, L1, p); /* log prior */
    A_xminusy(Ltilde,beta,mu_hattilde,e,p);
    xtx(e,&q,p);
    q *= -0.5;
    dpdq = lpz+lprz-lp-lpr + q - qz;
    probjump = (dpdq>0.0) ? 1.0 : exp(dpdq);
    duniform(1,&u);
    if (u<probjump){
     /* *n_jump += 1;*/
      y_assgn_x(beta,bnew,p);
      lp = lpz;
      y_assgn_x(mu_hat,mu_hattilde,p);
      B_assgn_A(L,Ltilde,p);
    }
  }
	matrix_times_vector(x, beta, res,n,p); 
  /* free memory */
  free_dvector(mu_hat,0,p-1);
  free_dvector(mu_hattilde,0,p-1);
  free_dvector(e,0,p-1);
  free_dvector(bnew,0,p-1);

  free_dmatrix(L1, 0,p-1,0,p-1);
  free_dmatrix(L, 0,p-1,0,p-1);
  free_dmatrix(Ltilde, 0,p-1,0,p-1);
  free_dmatrix(V, 0,p-1,0,p-1);

}

void dummy()
{
  printf("\n i'm the dummy \n");
}
